@@ -45,6 +45,11 @@ struct focusData
 	int focus_sharpness;
 	int sharpLeft;
 	int sharpRight;
+    int id;
+	int f_max;
+	int f_min;
+	int f_step;
+	int i_step;
 	int focusDir;
 	int arr_sharp[MAX_ARR_S];
 	int arr_foc[MAX_ARR_S];
@@ -54,7 +59,7 @@ struct focusData
 	int focus_wait;
 };
 
-void initFocusData (struct focusData *AFdata);
+struct focusData *initFocusData (int f_max, int f_min, int step, int id);
 
 int getSharpness (BYTE* img, int width, int height, int t);
 
