@@ -39,6 +39,7 @@
 #include <time.h>
 #include <glib.h>
 #include <glib/gstdio.h>
+#include <glib/gprintf.h>
 #include "config.h"
 #include "avilib.h"
 #include "defs.h"
@@ -121,7 +122,7 @@ static unsigned long str2ulong(BYTE *str)
 static int avi_sampsize(struct avi_t *AVI, int j)
 {
 	int s;
-	if (AVI->track[j].a_fmt == ISO_FORMAT_MPEG12) 
+	if (AVI->track[j].a_fmt != WAVE_FORMAT_PCM) 
 	{
 		s = 4;
 	} 
@@ -439,7 +440,7 @@ static int avi_update_header(struct avi_t *AVI)
        
 		// ThOe /4
 		OUTLONG(sampsize/4);      /* Scale */
-		OUTLONG(1000*AVI->track[j].mpgrate/8);
+		OUTLONG(AVI->track[j].mpgrate/8);
 		OUTLONG(0);             /* Start */
 		OUTLONG(4*AVI->track[j].audio_bytes/sampsize);   /* Length */
 		OUTLONG(0);             /* SuggestedBufferSize */
@@ -461,7 +462,7 @@ static int avi_update_header(struct avi_t *AVI)
 		OUTSHRT(AVI->track[j].a_chans);         /* Number of channels */
 		OUTLONG(AVI->track[j].a_rate);          /* SamplesPerSec */
 		// ThOe
-		OUTLONG(1000*AVI->track[j].mpgrate/8);
+		OUTLONG(AVI->track[j].mpgrate/8);
 		//ThOe (/4)
        
 		OUTSHRT(sampsize/4);           /* BlockAlign */
@@ -564,7 +565,9 @@ int AVI_open_output_file(struct avi_t *AVI, const char * filename)
 	i = avi_write(AVI->fdes,AVI_header,HEADERBYTES);
 	if (i != HEADERBYTES)
 	{
-		close(AVI->fdes);
+		
+		if(fsync(AVI->fdes) || close(AVI->fdes))
+			perror("AVI ERROR: couldn't write to avi file\n");
 		AVI_errno = AVI_ERR_WRITE;
 		g_mutex_free(AVI->mutex);
 		//free(AVI);
@@ -626,10 +629,10 @@ void AVI_set_audio(struct avi_t *AVI, int channels, long rate, int mpgrate, int
    
 		AVI->track[AVI->aptr].audio_bytes = 0;
 		AVI->track[AVI->aptr].a_chans = channels;
-		AVI->track[AVI->aptr].a_rate  = rate;
+		AVI->track[AVI->aptr].a_rate  = rate; /*sample rate*/
 		AVI->track[AVI->aptr].a_bits  = bits;
 		AVI->track[AVI->aptr].a_fmt   = format;
-		AVI->track[AVI->aptr].mpgrate = mpgrate;
+		AVI->track[AVI->aptr].mpgrate = mpgrate; /* bit rate in b/s */
 
 		avi_update_header(AVI);
 	g_mutex_unlock(AVI->mutex);
@@ -835,8 +838,8 @@ static int avi_close_output_file(struct avi_t *AVI)
 		} 
 		else
 		{
-			avgbsec = 1000*AVI->track[j].mpgrate/8;
-			scalerate = 1000*AVI->track[j].mpgrate/8;
+			avgbsec = AVI->track[j].mpgrate/8;
+			scalerate = AVI->track[j].mpgrate/8;
 		}
 
 		OUT4CC ("LIST");
@@ -860,7 +863,7 @@ static int avi_close_output_file(struct avi_t *AVI)
 		OUTLONG(0);             /* InitialFrames */
 	 
 		// VBR 
-		if (AVI->track[j].a_fmt == ISO_FORMAT_MPEG12 && AVI->track[j].a_vbr) 
+		if ((AVI->track[j].a_fmt != WAVE_FORMAT_PCM) && AVI->track[j].a_vbr) 
 		{
 			OUTLONG(nBlockAlign);                   /* Scale */
 			OUTLONG(AVI->track[j].a_rate);          /* Rate */
@@ -893,7 +896,7 @@ static int avi_close_output_file(struct avi_t *AVI)
 
 		OUT4CC ("strf");
 
-		if (AVI->track[j].a_fmt == ISO_FORMAT_MPEG12 && AVI->track[j].a_vbr) 
+		if ((AVI->track[j].a_fmt != WAVE_FORMAT_PCM) && AVI->track[j].a_vbr) 
 		{
 
 			OUTLONG(30);                            /* # of bytes to follow */ // mplayer writes 28
@@ -901,7 +904,7 @@ static int avi_close_output_file(struct avi_t *AVI)
 			OUTSHRT(AVI->track[j].a_chans);         /* Number of channels */      // 2
 			OUTLONG(AVI->track[j].a_rate);          /* SamplesPerSec */           // 4
 			//ThOe/tibit
-			OUTLONG(1000*AVI->track[j].mpgrate/8);  /* maybe we should write an avg. */ // 4
+			OUTLONG(AVI->track[j].mpgrate/8);  /* maybe we should write an avg. */ // 4
 			OUTSHRT(nBlockAlign);                   /* BlockAlign */              // 2
 			OUTSHRT(AVI->track[j].a_bits);          /* BitsPerSample */           // 2
 
@@ -912,14 +915,14 @@ static int avi_close_output_file(struct avi_t *AVI)
 			OUTSHRT(1);                             /* nFramesPerBlock */          // 2
 			OUTSHRT(0);                             /* nCodecDelay */              // 2
 		} 
-		else if (AVI->track[j].a_fmt == ISO_FORMAT_MPEG12 && !AVI->track[j].a_vbr) 
+		else if ((AVI->track[j].a_fmt != WAVE_FORMAT_PCM) && !AVI->track[j].a_vbr) 
 		{
 			OUTLONG(30);                            /* # of bytes to follow (30)*/
 			OUTSHRT(AVI->track[j].a_fmt);           /* Format */
 			OUTSHRT(AVI->track[j].a_chans);         /* Number of channels */
 			OUTLONG(AVI->track[j].a_rate);          /* SamplesPerSec */
 			//ThOe/tibit
-			OUTLONG(1000*AVI->track[j].mpgrate/8);
+			OUTLONG(AVI->track[j].mpgrate/8);
 			OUTSHRT(sampsize/4);                    /* BlockAlign */
 			OUTSHRT(AVI->track[j].a_bits);          /* BitsPerSample */
 
@@ -1222,8 +1225,9 @@ int AVI_close(struct avi_t *AVI)
 		else
 			ret = 0;
 
-		/* Even if there happened a error, we first clean up */
-		close(AVI->fdes);
+		/* Even if a error ocurred, we first clean up */
+		if(fsync(AVI->fdes) || close(AVI->fdes))
+			perror("AVI ERROR: couldn't write avi data\n");
 		if(AVI->idx) free(AVI->idx);
 		if(AVI->video_index) free(AVI->video_index);
 		for (j=0; j<AVI->anum; j++) 
