@@ -253,11 +253,14 @@ struct avi_t
 #define WAVE_FORMAT_YAMAHA_ADPCM        (0x0020)
 #define WAVE_FORMAT_DSP_TRUESPEECH      (0x0022)
 #define WAVE_FORMAT_GSM610              (0x0031)
-#define IBM_FORMAT_MULAW                (0x0101)
-#define IBM_FORMAT_ALAW                 (0x0102)
-#define IBM_FORMAT_ADPCM                (0x0103)
+#define WAVE_FORMAT_MP3                 (0x0055)
+#define WAVE_FORMAT_MPEG12              (0x0050)
+#define WAVE_FORMAT_AAC                 (0x00ff)
+#define WAVE_FORMAT_IBM_MULAW           (0x0101)
+#define WAVE_FORMAT_IBM_ALAW            (0x0102)
+#define WAVE_FORMAT_IBM_ADPCM           (0x0103)
+#define WAVE_FORMAT_AC3                 (0x2000)
 /*extra audio formats (codecs)*/
-#define DOLBY_FORMAT_AC3 		(0x2000)
 #define ANTEX_FORMAT_ADPCME		(0x0033)
 #define AUDIO_FORMAT_APTX		(0x0025)
 #define AUDIOFILE_FORMAT_AF10		(0x0026)
@@ -280,8 +283,6 @@ struct avi_t
 #define DOLBY_FORMAT_AC3_SPDIF		(0x0092)
 #define ESS_FORMAT_ESPCM		(0x0061)
 #define IEEE_FORMAT_FLOAT		(0x0003)
-#define ISO_FORMAT_MP3			(0x0055)
-#define ISO_FORMAT_MPEG12		(0x0050)
 #define MS_FORMAT_MSAUDIO1_DIVX		(0x0160)
 #define MS_FORMAT_MSAUDIO2_DIVX		(0x0161)
 #define OGG_FORMAT_VORBIS1		(0x674f)
