@@ -26,17 +26,19 @@
 #include <glib/gi18n.h>
 #include <glib.h>
 #include <glib/gprintf.h>
+#include <gdk/gdkkeysyms.h>
 
 #include <portaudio.h>
 
 #include "v4l2uvc.h"
+#include "v4l2_dyna_ctrls.h"
 #include "avilib.h"
 #include "globals.h"
 #include "sound.h"
 #include "ms_time.h"
 #include "string_utils.h"
 #include "video.h"
-#include "mp2.h"
+#include "acodecs.h"
 #include "profile.h"
 #include "close.h"
 #include "timers.h"
@@ -45,18 +47,37 @@
 #include "lavc_common.h"
 #include "create_video.h"
 #include "video_format.h"
-/*---------------------------- error message dialog-----------------------------*/
+
+/*--------------------------- warning message dialog ----------------------------*/
+void
+WARN_DIALOG(const char *warn_title, const char* warn_msg, struct ALL_DATA *all_data)
+{
+	struct GWIDGET *gwidget = all_data->gwidget;
+	
+	GtkWidget *warndialog;
+	warndialog = gtk_message_dialog_new (GTK_WINDOW(gwidget->mainwin),
+		GTK_DIALOG_DESTROY_WITH_PARENT,
+		GTK_MESSAGE_WARNING,
+		GTK_BUTTONS_CLOSE,
+		"%s",gettext(warn_title));
+
+	gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(warndialog),
+		"%s",gettext(warn_msg));
+
+	gtk_widget_show(warndialog);
+	gtk_dialog_run (GTK_DIALOG (warndialog));
+	gtk_widget_destroy (warndialog);
+}
+
+/*---------------------------- error message dialog -----------------------------*/
 void 
 ERR_DIALOG(const char *err_title, const char* err_msg, struct ALL_DATA *all_data)
 {
 	struct GWIDGET *gwidget = all_data->gwidget;
-	struct VidState *s = all_data->s;
-	struct paRecordData *pdata = all_data->pdata;
 	struct GLOBAL *global = all_data->global;
-	struct focusData *AFdata = all_data->AFdata;
-	struct vdIn *videoIn = all_data->videoIn;
-	struct VideoFormatData *videoF = all_data->videoF;
-
+	
+	gboolean control_only = (global->control_only || global->add_ctrls);
+	
 	GtkWidget *errdialog;
 	errdialog = gtk_message_dialog_new (GTK_WINDOW(gwidget->mainwin),
 		GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -71,35 +92,20 @@ ERR_DIALOG(const char *err_title, const char* err_msg, struct ALL_DATA *all_data
 	gtk_dialog_run (GTK_DIALOG (errdialog));
 	gtk_widget_destroy (errdialog);
 
-	g_free(gwidget);
-	gwidget = NULL;
-	all_data->gwidget = NULL;
-
-	g_free(AFdata);
-	AFdata = NULL;
-	all_data->AFdata = NULL;
-
-	g_free(s);
-	s = NULL;
-	all_data->s = NULL;
-
-	if (global) closeGlobals (global);
-	global = NULL;
-	all_data->global = NULL;
-
-	g_free(pdata);
-	pdata = NULL;
-	all_data->pdata = NULL;    
-
-	if(videoIn->fd > 0) close_v4l2(videoIn);
-	else g_free(videoIn);
-	videoIn=NULL;
-	all_data->videoIn = NULL;
-
-	g_free(videoF);
-	videoF = NULL;
-	all_data->videoF = NULL;
+	clean_struct(all_data);
 
+	/* error dialog is allways called before creating the main loop */
+	/* so no need for gtk_main_quit()                               */
+	/* but this means we must close portaudio before exiting        */
+	if(!control_only)
+	{
+		g_printf("Closing portaudio ...");
+		if (Pa_Terminate() != paNoError) 
+			g_printf("Error\n");
+		else
+			g_printf("OK\n");
+	}
+	
 	g_printf("Terminated.\n");;
 	exit(1);
 };
@@ -116,6 +122,7 @@ delete_event (GtkWidget *widget, GdkEventConfigure *event, void *data)
 	return 0;
 }
 
+/*--------------------------- controls enable/disable --------------------------*/
 /*image controls*/
 void 
 set_sensitive_img_contrls (const int flag, struct GWIDGET *gwidget)
@@ -126,36 +133,135 @@ set_sensitive_img_contrls (const int flag, struct GWIDGET *gwidget)
 	gtk_widget_set_sensitive(gwidget->ImageInc, flag);/*image inc checkbox*/
 }
 
+/* sound controls*/
+void 
+set_sensitive_snd_contrls (const int flag, struct GWIDGET *gwidget)
+{
+	gtk_widget_set_sensitive (gwidget->SndSampleRate, flag);
+	gtk_widget_set_sensitive (gwidget->SndDevice, flag);
+	gtk_widget_set_sensitive (gwidget->SndNumChan, flag);
+	gtk_widget_set_sensitive (gwidget->SndComp, flag);
+}
+
+/*video controls*/
+void 
+set_sensitive_vid_contrls (const int flag, const int sndEnable, struct GWIDGET *gwidget)
+{
+	/* sound and video compression controls */
+	gtk_widget_set_sensitive (gwidget->VidCodec, flag);
+	gtk_widget_set_sensitive (gwidget->SndEnable, flag); 
+	gtk_widget_set_sensitive (gwidget->VidInc, flag);
+	gtk_widget_set_sensitive (gwidget->VidFormat, flag);/*video format combobox*/
+	/* resolution and input format combos   */
+	gtk_widget_set_sensitive (gwidget->Resolution, flag);
+	gtk_widget_set_sensitive (gwidget->InpType, flag);
+	gtk_widget_set_sensitive (gwidget->FrameRate, flag);
+	/* Video File entry and open button     */
+	gtk_widget_set_sensitive (gwidget->VidFNameEntry, flag);
+	gtk_widget_set_sensitive (gwidget->VidFileButt, flag);
+	
+	if(sndEnable > 0) 
+	{
+		set_sensitive_snd_contrls(flag, gwidget);
+	}
+    
+	gwidget->vid_widget_state = flag;
+}
+
+gboolean
+key_pressed (GtkWidget *win, GdkEventKey *event, struct ALL_DATA *all_data)
+{
+    struct VidState *s = all_data->s;
+    struct GWIDGET *gwidget = all_data->gwidget;
+    struct vdIn *videoIn = all_data->videoIn;
+    //struct GLOBAL *global = all_data->global;
+    /* If we have modifiers, and either Ctrl, Mod1 (Alt), or any
+     * of Mod3 to Mod5 (Mod2 is num-lock...) are pressed, we
+     * let Gtk+ handle the key */
+    //printf("camera key pressed (key:%i)\n", event->keyval);
+    if (event->state != 0
+            && ((event->state & GDK_CONTROL_MASK)
+            || (event->state & GDK_MOD1_MASK)
+            || (event->state & GDK_MOD3_MASK)
+            || (event->state & GDK_MOD4_MASK)
+            || (event->state & GDK_MOD5_MASK)))
+        return FALSE;
+    
+    if(videoIn->PanTilt)
+    {
+        switch (event->keyval)
+        {
+            case GDK_Down:
+            case GDK_KP_Down:
+                /*Tilt Down*/
+                uvcPanTilt (videoIn->fd, s->control_list, 0, 1);
+                return TRUE;
+                
+            case GDK_Up:
+            case GDK_KP_Up:
+                /*Tilt UP*/
+                uvcPanTilt (videoIn->fd, s->control_list, 0, -1);
+                return TRUE;
+                
+            case GDK_Left:
+            case GDK_KP_Left:
+                /*Pan Left*/
+                uvcPanTilt (videoIn->fd, s->control_list, 1, 1);
+                return TRUE;
+                
+            case GDK_Right:
+            case GDK_KP_Right:
+                /*Pan Right*/
+                uvcPanTilt (videoIn->fd, s->control_list, 1, -1);
+                return TRUE;
+                
+            default:
+                break;
+        }
+    }
+    
+    switch (event->keyval)
+    {
+        case GDK_WebCam:
+
+        /* camera button pressed - trigger image capture*/
+        gtk_button_clicked (GTK_BUTTON(gwidget->CapImageButt));
+        return TRUE;
+    }
+
+    return FALSE;
+}
+
+
 /*----------------------------- Callbacks ------------------------------------*/
 /*slider controls callback*/
 void
 slider_changed (GtkRange * range, struct ALL_DATA *all_data)
 {
-	struct VidState *s = all_data->s;
-	struct GLOBAL *global = all_data->global;
-	struct vdIn *videoIn = all_data->videoIn;
+    struct VidState *s = all_data->s;
+    struct GLOBAL *global = all_data->global;
+    struct vdIn *videoIn = all_data->videoIn;
 
-	ControlInfo * ci = g_object_get_data (G_OBJECT (range), "control_info");
-	InputControl * c = s->control + ci->idx;
-	int val = (int) gtk_range_get_value (range);
+    int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (range), "control_info"));
+    Control *c = get_ctrl_by_id(s->control_list, id);
+    
+    int val = (int) gtk_range_get_value (range);
+    
+    c->value = val;
 
-	if (input_set_control (videoIn->fd, c, val) == 0) 
-	{
-		gtk_spin_button_set_value(GTK_SPIN_BUTTON(ci->spinbutton), val);
-	}
-	else 
-	{
-		if (global->debug) g_printerr ("%s change to %d failed\n",c->name, val);
-		if (input_get_control (videoIn->fd, c, &val) == 0) 
-		{
-			if (global->debug) g_printerr ("hardware value is %d\n", val);
-			gtk_range_set_value (GTK_RANGE(ci->widget),val);
-		}
-		else 
-		{
-			g_printerr ("hardware get failed\n");
-		}
-	}
+    set_ctrl(videoIn->fd, s->control_list, id);
+    
+    //update spin
+    if(c->spinbutton)
+    {   
+        //disable widget signals
+        g_signal_handlers_block_by_func(GTK_SPIN_BUTTON(c->spinbutton), 
+            G_CALLBACK (spin_changed), all_data); 
+        gtk_spin_button_set_value (GTK_SPIN_BUTTON(c->spinbutton), c->value);
+        //enable widget signals
+        g_signal_handlers_unblock_by_func(GTK_SPIN_BUTTON(c->spinbutton), 
+            G_CALLBACK (spin_changed), all_data);
+    }
 	
 	s = NULL;
 	global = NULL;
@@ -170,27 +276,25 @@ spin_changed (GtkSpinButton * spin, struct ALL_DATA *all_data)
 	struct GLOBAL *global = all_data->global;
 	struct vdIn *videoIn = all_data->videoIn;
 	
-	ControlInfo * ci = g_object_get_data (G_OBJECT (spin), "control_info");
-	InputControl * c = s->control + ci->idx;
+	int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (spin), "control_info"));
+	Control *c = get_ctrl_by_id(s->control_list, id);
+    
 	int val = gtk_spin_button_get_value_as_int (spin);
+    c->value = val;
+
+    set_ctrl(videoIn->fd, s->control_list, id);
+    
+    if(c->widget)
+    {
+        //disable widget signals
+        g_signal_handlers_block_by_func(GTK_SCALE (c->widget), 
+            G_CALLBACK (slider_changed), all_data);
+        gtk_range_set_value (GTK_RANGE (c->widget), c->value);
+        //enable widget signals    
+        g_signal_handlers_unblock_by_func(GTK_SCALE (c->widget), 
+            G_CALLBACK (slider_changed), all_data);
+    }
 
-	if (input_set_control (videoIn->fd, c, val) == 0) 
-	{
-		gtk_range_set_value (GTK_RANGE(ci->widget),val);
-	}
-	else 
-	{
-		if (global->debug) g_printerr ("%s change to %d failed\n",c->name, val);
-		if (input_get_control (videoIn->fd, c, &val) == 0) 
-		{
-			if (global->debug) g_printerr ("hardware value is %d\n", val);
-			gtk_spin_button_set_value(GTK_SPIN_BUTTON(ci->spinbutton),val);
-		}
-		else 
-		{
-			g_printerr ("hardware get failed\n");
-		}
-	}
 	s = NULL;
 	global = NULL;
 	videoIn = NULL;
@@ -214,30 +318,31 @@ set_jpeg_comp_clicked (GtkButton * jpeg_comp, struct ALL_DATA *all_data)
 void
 autofocus_changed (GtkToggleButton * toggle, struct ALL_DATA *all_data) 
 {
+    struct VidState *s = all_data->s;
 	struct GLOBAL *global = all_data->global;
 	struct focusData *AFdata = all_data->AFdata;
 	struct vdIn *videoIn = all_data->videoIn;
 	
-	ControlInfo * ci = g_object_get_data (G_OBJECT (toggle), "control_info");
-	int val;
+	//int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (toggle), "control_info"));
+	Control *c = get_ctrl_by_id(s->control_list, AFdata->id);
 	
-	val = gtk_toggle_button_get_active (toggle) ? 1 : 0;
+	int val = gtk_toggle_button_get_active (toggle) ? 1 : 0;
 
 	/*if autofocus disable manual focus control*/
-	gtk_widget_set_sensitive (ci->widget, !val);
-	gtk_widget_set_sensitive (ci->spinbutton, !val);
+	gtk_widget_set_sensitive (c->widget, !val);
+	gtk_widget_set_sensitive (c->spinbutton, !val);
 
 	/*reset flag*/
 	AFdata->flag = 0;
 	AFdata->ind = 0;
 	AFdata->focus = -1; /*reset focus*/
-	AFdata->right = 255;
-	AFdata->left = 8;
+	AFdata->right = AFdata->f_max;
+	AFdata->left = AFdata->i_step;
 	/*set focus to first value if autofocus enabled*/
 	if (val>0) 
 	{
-		if (set_focus (videoIn->fd, AFdata->focus) != 0) 
-			g_printerr("ERROR: couldn't set focus to %d\n", AFdata->focus);
+	    c->value = AFdata->focus;
+	    set_ctrl(videoIn->fd, s->control_list, AFdata->id);
 	}
 	global->autofocus = val;
 
@@ -250,91 +355,31 @@ autofocus_changed (GtkToggleButton * toggle, struct ALL_DATA *all_data)
 void
 check_changed (GtkToggleButton * toggle, struct ALL_DATA *all_data)
 {
-	struct VidState *s = all_data->s;
-	struct GLOBAL *global = all_data->global;
-	struct vdIn *videoIn = all_data->videoIn;
+    struct VidState *s = all_data->s;
+    struct GLOBAL *global = all_data->global;
+    struct vdIn *videoIn = all_data->videoIn;
 
-	ControlInfo * ci = g_object_get_data (G_OBJECT (toggle), "control_info");
-	InputControl * c = s->control + ci->idx;
-	int val;
+    int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (toggle), "control_info"));
+    Control *c = get_ctrl_by_id(s->control_list, id);
+    
+    int val = gtk_toggle_button_get_active (toggle) ? 1 : 0;
 
-	val = gtk_toggle_button_get_active (toggle) ? 1 : 0;
+    c->value = val;
 
-	if (input_set_control (videoIn->fd, c, val) != 0)
-	{
-		g_printerr ("%s change to %d failed\n",c->name, val);
-		if (input_get_control (videoIn->fd, c, &val) == 0) 
-		{
-			g_printerr ("hardware value is %d\n", val);
-		}
-		else 
-		{
-			g_printerr ("hardware get failed\n");
-		}
-	} 
-	else 
-	{
-		if (global->debug) g_printf("changed %s to %d\n",c->name,val);
-		if (input_get_control (videoIn->fd, c, &val) == 0) 
-		{
-			if (global->debug) g_printf ("hardware value is %d\n", val);
-		}
-		else 
-		{
-			g_printerr ("hardware get failed\n");
-		}
-		
-	}
-	
-	s = NULL;
-	global = NULL;
-	videoIn = NULL;
-}
-
-void
-bayer_changed (GtkToggleButton * toggle, struct ALL_DATA *all_data)
-{
-	struct VidState *s = all_data->s;
-	struct GLOBAL *global = all_data->global;
-	struct vdIn *videoIn = all_data->videoIn;
-	
-	ControlInfo * ci = g_object_get_data (G_OBJECT (toggle), "control_info");
-	InputControl * c = s->control + ci->idx;
-	int val;
-
-	val = gtk_toggle_button_get_active (toggle) ? 1 : 0;
-	if (input_set_control (videoIn->fd, c, val) != 0) 
-	{
-		g_printerr ("%s change to %d failed\n",c->name, val);
-		if (input_get_control (videoIn->fd, c, &val) == 0) 
-		{
-			g_printerr ("hardware value is %d\n", val);
-		}
-		else 
-		{
-			g_printerr ("hardware get failed\n");
-		}
-	}
-	else
-	{
-		if (global->debug) g_printf("changed %s to %d\n",c->name,val);
-		/*stop and restart stream*/
-		videoIn->setFPS=1;
-		/*read value*/
-		if (input_get_control (videoIn->fd, c, &val) == 0) 
-		{
-			if (val>0) videoIn->isbayer=1;
-			else videoIn->isbayer=0;
-		}
-		else 
-		{
-			g_printerr ("hardware get failed\n");
-		}
-	}
-
-	s = NULL;
-	global = NULL;
-	videoIn = NULL;
+    set_ctrl(videoIn->fd, s->control_list, id);
+    
+    if(id == V4L2_CID_DISABLE_PROCESSING_LOGITECH)
+    {
+        if (c->value > 0) videoIn->isbayer=1;
+        else videoIn->isbayer=0;
+        
+        //restart stream by changing fps
+        videoIn->setFPS = 1;
+    }
+    
+    s = NULL;
+    global = NULL;
+    videoIn = NULL;
 }
 
 void
@@ -348,19 +393,6 @@ pix_ord_changed (GtkComboBox * combo, struct ALL_DATA *all_data)
 	videoIn=NULL;
 }
 
-void
-reversePan_changed (GtkToggleButton * toggle, struct ALL_DATA *all_data)
-{
-	struct GLOBAL *global = all_data->global;
-
-	int val;
-
-	val = gtk_toggle_button_get_active (toggle) ? -1 : 1;
-
-	if(global->PanStep >0) global->PanStep= val * global->PanStep;
-	else global->PanStep= -val * global->PanStep;
-}
-
 /*combobox controls callback*/
 void
 combo_changed (GtkComboBox * combo, struct ALL_DATA *all_data)
@@ -368,37 +400,62 @@ combo_changed (GtkComboBox * combo, struct ALL_DATA *all_data)
 	struct VidState *s = all_data->s;
 	struct vdIn *videoIn = all_data->videoIn;
 
-	ControlInfo * ci = g_object_get_data (G_OBJECT (combo), "control_info");
-	InputControl * c = s->control + ci->idx;
-	int index = gtk_combo_box_get_active (combo);
-	int val=0;
-
-	if (c->id == V4L2_CID_EXPOSURE_AUTO_OLD) 
-	{
-		val=exp_vals[videoIn->available_exp[index]];
-	} 
-	else 
-	{
-		val=index;
-	}
-
-	if (input_set_control (videoIn->fd, c, val) != 0) 
-	{
-		g_printerr ("%s change to %d failed\n",c->name, val);
-		if (input_get_control (videoIn->fd, c, &val) == 0) 
-		{
-			g_printerr ("hardware value is %d\n", val);
-		}
-		else 
-		{
-			g_printerr ("hardware get failed\n");
-		}
-	}
+    int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (combo), "control_info"));
+    Control *c = get_ctrl_by_id(s->control_list, id);
+    
+    int index = gtk_combo_box_get_active (combo);
+    c->value = index;
 
+    set_ctrl(videoIn->fd, s->control_list, id);
+    
 	s = NULL;
 	videoIn = NULL;
 }
 
+/* generic button control */
+void
+button_clicked (GtkButton * Button, struct ALL_DATA *all_data)
+{
+    struct VidState *s = all_data->s;
+	struct vdIn *videoIn = all_data->videoIn;
+
+    int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (Button), "control_info"));
+    Control *c = get_ctrl_by_id(s->control_list, id);
+    
+    c->value = 1;
+    set_ctrl(videoIn->fd, s->control_list, id);
+}
+
+/* Pan Tilt button 1 control */
+void
+button_PanTilt1_clicked (GtkButton * Button, struct ALL_DATA *all_data)
+{
+    struct VidState *s = all_data->s;
+	struct vdIn *videoIn = all_data->videoIn;
+
+    int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (Button), "control_info"));
+    Control *c = get_ctrl_by_id(s->control_list, id);
+    
+    int val = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(c->spinbutton));
+    c->value = val;
+    set_ctrl(videoIn->fd, s->control_list, id);
+}
+
+/* Pan Tilt button 2 control */
+void
+button_PanTilt2_clicked (GtkButton * Button, struct ALL_DATA *all_data)
+{
+    struct VidState *s = all_data->s;
+	struct vdIn *videoIn = all_data->videoIn;
+
+    int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (Button), "control_info"));
+    Control *c = get_ctrl_by_id(s->control_list, id);
+    
+    int val = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(c->spinbutton));
+    c->value = -val;
+    set_ctrl(videoIn->fd, s->control_list, id);
+}   
+
 /*video format control callback*/
 void
 VidFormat_changed (GtkComboBox * VidFormat, struct ALL_DATA *all_data) 
@@ -437,42 +494,24 @@ setfocus_clicked (GtkButton * FocusButton, struct ALL_DATA *all_data)
 {
 	struct focusData *AFdata = all_data->AFdata;
 	struct vdIn *videoIn = all_data->videoIn;
-
+    struct VidState *s = all_data->s;
+    
 	AFdata->setFocus = 1;
 	AFdata->ind = 0;
 	AFdata->flag = 0;
 	AFdata->right = 255;
 	AFdata->left = 8;
 	AFdata->focus = -1; /*reset focus*/
-	if (set_focus (videoIn->fd, AFdata->focus) != 0)
-		g_printerr("ERROR: couldn't set focus to %d\n", AFdata->focus);
+	
+    Control *c = get_ctrl_by_id(s->control_list, AFdata->id);
+	c->value = AFdata->focus;
+
+    set_ctrl(videoIn->fd, s->control_list, AFdata->id);
 
 	AFdata = NULL;
 	videoIn = NULL;
 }
 
-// Pan/Tilt (for motor cameras ex: Logitech Orbit/Sphere)
-void
-PanTilt_clicked (GtkButton * PanTilt, struct ALL_DATA *all_data)
-{
-	struct GLOBAL *global = all_data->global;
-	struct vdIn *videoIn = all_data->videoIn;
-	int pan = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (PanTilt), "pan_info"));
-	int tilt = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (PanTilt), "tilt_info"));
-	int reset = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (PanTilt), "reset_info"));
-	
-	if(uvcPanTilt(videoIn->fd, pan * (global->PanStep), tilt * (global->TiltStep), reset)<0) 
-	{
-		g_printerr("Pan/Tilt Error: Pan = %d; Tilt = %d; reset = %d\n",
-			pan * (global->PanStep),
-			tilt * (global->TiltStep),
-			reset);
-	}
-
-	global = NULL;
-	videoIn = NULL;
-}
-
 void 
 Devices_changed (GtkComboBox * Devices, struct ALL_DATA *all_data)
 {
@@ -545,10 +584,17 @@ resolution_changed (GtkComboBox * Resolution, struct ALL_DATA *all_data)
 	VidCap *listVidCap = NULL;
 	int current_format = videoIn->listFormats->current_format;
 	int cmb_index = gtk_combo_box_get_active(Resolution);
+	char temp_str[20];
+	
+	g_mutex_lock(videoIn->mutex);
+		gboolean capVid = videoIn->capVid;
+	g_mutex_unlock(videoIn->mutex);
+	/*disable fps combobox signals*/
+	g_signal_handlers_block_by_func(GTK_COMBO_BOX(gwidget->FrameRate), G_CALLBACK (FrameRate_changed), all_data);
+	/* clear out the old fps list... */
+	GtkListStore *store = GTK_LIST_STORE(gtk_combo_box_get_model (GTK_COMBO_BOX (gwidget->FrameRate)));
+	gtk_list_store_clear(store);
 
-	/* The new resolution is writen to conf file at exit             */
-	/* then is read back at start. This means that for changing      */
-	/* resolution we must restart the application                    */
 
 	listVidCap = &videoIn->listFormats->listVidFormats[current_format].listVidCap[cmb_index];
 	global->width = listVidCap->width;
@@ -557,50 +603,37 @@ resolution_changed (GtkComboBox * Resolution, struct ALL_DATA *all_data)
 	/*check if frame rate is available at the new resolution*/
 	int i=0;
 	int deffps=0;
-	 
-	for(i = 0 ; i < listVidCap->numb_frates; i++)
+	
+	for ( i = 0 ; i < listVidCap->numb_frates ; i++) 
 	{
-		if ((listVidCap->framerate_num[i] == global->fps_num) && 
-			(listVidCap->framerate_denom[i] == global->fps)) 
-		{
-				deffps = i;
-				break;
-		}
+		g_snprintf(temp_str,18,"%i/%i fps", listVidCap->framerate_denom[i],
+			listVidCap->framerate_num[i]);
+		gtk_combo_box_append_text(GTK_COMBO_BOX(gwidget->FrameRate),temp_str);
+		
+		if (( global->fps_num == listVidCap->framerate_num[i]) && 
+			(global->fps == listVidCap->framerate_denom[i]))
+				deffps=i;//set selected
 	}
+
+	/*set default fps in combo*/
+	gtk_combo_box_set_active(GTK_COMBO_BOX(gwidget->FrameRate),deffps);
+	
+	/*enable fps combobox signals*/ 
+	g_signal_handlers_unblock_by_func(GTK_COMBO_BOX(gwidget->FrameRate), G_CALLBACK (FrameRate_changed), all_data);
 	
 	if (listVidCap->framerate_num)	
 		global->fps_num = listVidCap->framerate_num[deffps];
 
 	if (listVidCap->framerate_denom)
 		global->fps = listVidCap->framerate_denom[deffps];
-	
-	gwidget->restartdialog = gtk_dialog_new_with_buttons (_("Program Restart"),
-		GTK_WINDOW(gwidget->mainwin),
-		GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-		_("now"),
-		GTK_RESPONSE_ACCEPT,
-		_("Later"),
-		GTK_RESPONSE_REJECT,
-		NULL);
-	
-	GtkWidget *message = gtk_label_new (_("Changes will only take effect after guvcview restart.\n\n\nRestart now?\n"));
-	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(gwidget->restartdialog)->vbox), message);
-	gtk_widget_show_all(GTK_WIDGET(GTK_CONTAINER (GTK_DIALOG(gwidget->restartdialog)->vbox)));
-	
-	gint result = gtk_dialog_run (GTK_DIALOG (gwidget->restartdialog));
-	switch (result) 
-	{
-		case GTK_RESPONSE_ACCEPT:
-			/*restart app*/
-			shutd(1, all_data);
-			break;
 
-		default:
-			/* do nothing since Restart rejected*/
-			break;
+	if(capVid)
+	{
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(gwidget->CapVidButt), FALSE);
+		gdk_flush();
 	}
 
-	gtk_widget_destroy (gwidget->restartdialog);
+	global->change_res = TRUE;
 
 	gwidget = NULL;
 	global = NULL;
@@ -609,74 +642,58 @@ resolution_changed (GtkComboBox * Resolution, struct ALL_DATA *all_data)
 
 /* Input Format control */
 void
-ImpType_changed(GtkComboBox * ImpType, struct ALL_DATA *all_data) 
+InpType_changed(GtkComboBox * InpType, struct ALL_DATA *all_data) 
 {
 	struct GWIDGET *gwidget = all_data->gwidget;
 	struct GLOBAL *global = all_data->global;
 	struct vdIn *videoIn = all_data->videoIn;
 	
-	int index = gtk_combo_box_get_active(ImpType);
+	int format = 0;
+	char temp_str[20];
+	int index = gtk_combo_box_get_active(InpType);
+	int i=0;
+	//int j=0;
+	int defres = 0;
+	VidFormats *listVidFormats;
 	
+	/*disable resolution combobox signals*/
+	g_signal_handlers_block_by_func(GTK_COMBO_BOX(gwidget->Resolution), G_CALLBACK (resolution_changed), all_data);
+	
+	/* clear out the old resolution list... */
+	GtkListStore *store = GTK_LIST_STORE(gtk_combo_box_get_model (GTK_COMBO_BOX (gwidget->Resolution)));
+	gtk_list_store_clear(store);
 	
 	videoIn->listFormats->current_format = index;
+	listVidFormats = &videoIn->listFormats->listVidFormats[index];
 	
-	/*check if frame rate and resolution are available   */
-	/*if not use minimum values - defres=0 and deffps=0  */
-	int i=0;
-	int j=0;
-	int defres=0;
-	int deffps=0;
+	format = videoIn->listFormats->listVidFormats[videoIn->listFormats->current_format].format;
+	get_PixMode(format, global->mode);
 	
-	
-	for (i=0;i<videoIn->listFormats->listVidFormats[videoIn->listFormats->current_format].numb_res;i++) 
+	/*redraw resolution combo for new format*/
+	for(i = 0 ; i < listVidFormats->numb_res ; i++)  
 	{
-		if((videoIn->listFormats->listVidFormats[videoIn->listFormats->current_format].listVidCap[i].height==global->height) &&
-			(videoIn->listFormats->listVidFormats[videoIn->listFormats->current_format].listVidCap[i].width==global->width) ) 
+		if (listVidFormats->listVidCap[i].width>0)
 		{
-			/* resolution ok check fps*/
-			defres=i;
-			for (j=0;j<videoIn->listFormats->listVidFormats[videoIn->listFormats->current_format].listVidCap[i].numb_frates;j++) 
-			{
-				if ((videoIn->listFormats->listVidFormats[videoIn->listFormats->current_format].listVidCap[i].framerate_num[j]==global->fps_num) && 
-					(videoIn->listFormats->listVidFormats[videoIn->listFormats->current_format].listVidCap[i].framerate_denom[j]==global->fps))
-						deffps=j;
-			}
+			g_snprintf(temp_str,18,"%ix%i", listVidFormats->listVidCap[i].width,
+							 listVidFormats->listVidCap[i].height);
+			gtk_combo_box_append_text(GTK_COMBO_BOX(gwidget->Resolution),temp_str);
+			
+			if ((global->width == listVidFormats->listVidCap[i].width) && 
+				(global->height == listVidFormats->listVidCap[i].height))
+					defres=i;//set selected resolution index
 		}
 	}
-	global->format = videoIn->listFormats->listVidFormats[videoIn->listFormats->current_format].format;
-	get_PixMode(global->format, global->mode);
-	global->height=videoIn->listFormats->listVidFormats[videoIn->listFormats->current_format].listVidCap[defres].height;
-	global->width=videoIn->listFormats->listVidFormats[videoIn->listFormats->current_format].listVidCap[defres].width;
-	global->fps_num=videoIn->listFormats->listVidFormats[videoIn->listFormats->current_format].listVidCap[defres].framerate_num[deffps];
-	global->fps=videoIn->listFormats->listVidFormats[videoIn->listFormats->current_format].listVidCap[defres].framerate_denom[deffps];
 	
-	gwidget->restartdialog = gtk_dialog_new_with_buttons (_("Program Restart"),
-		GTK_WINDOW(gwidget->mainwin),
-		GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
-		_("now"),
-		GTK_RESPONSE_ACCEPT,
-		_("Later"),
-		GTK_RESPONSE_REJECT,
-		NULL);
 	
-	GtkWidget *message = gtk_label_new (_("Changes will only take effect after guvcview restart.\n\n\nRestart now?\n"));
-	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(gwidget->restartdialog)->vbox), message);
-	gtk_widget_show_all(GTK_WIDGET(GTK_CONTAINER (GTK_DIALOG(gwidget->restartdialog)->vbox)));
+	global->height = listVidFormats->listVidCap[defres].height;
+	global->width = listVidFormats->listVidCap[defres].width;
+	global->format = format;
 	
-	gint result = gtk_dialog_run (GTK_DIALOG (gwidget->restartdialog));
-	switch (result) 
-	{
-		case GTK_RESPONSE_ACCEPT:
-			/*restart app*/
-			shutd(1, all_data);
-			break;
-			
-		default:
-			/* do nothing since Restart rejected*/
-			break;
-	}
-  
-	gtk_widget_destroy (gwidget->restartdialog);
+	/*enable resolution combobox signals*/
+	g_signal_handlers_unblock_by_func(GTK_COMBO_BOX(gwidget->Resolution), G_CALLBACK (resolution_changed), all_data);
+	
+	/*reset resolution/format*/
+	gtk_combo_box_set_active(GTK_COMBO_BOX(gwidget->Resolution),defres);
 	
 	gwidget = NULL;
 	global = NULL;
@@ -695,13 +712,13 @@ FrameRate_changed (GtkComboBox * FrameRate, struct ALL_DATA *all_data)
 	
 	int index = gtk_combo_box_get_active (FrameRate);
 		
-	videoIn->fps=videoIn->listFormats->listVidFormats[videoIn->listFormats->current_format].listVidCap[resind].framerate_denom[index];
-	videoIn->fps_num=videoIn->listFormats->listVidFormats[videoIn->listFormats->current_format].listVidCap[resind].framerate_num[index];
+	global->fps=videoIn->listFormats->listVidFormats[videoIn->listFormats->current_format].listVidCap[resind].framerate_denom[index];
+	global->fps_num=videoIn->listFormats->listVidFormats[videoIn->listFormats->current_format].listVidCap[resind].framerate_num[index];
  
+	/*fps change is done in two fases:
+	 * 1- in video.c we try to change device fps (and set flag to 2)
+	 * 2- in v4l2uvc.c (uvcGrab) we query and queue the buffers if using MMAP (and reset flag to 0)*/
 	videoIn->setFPS=1;
-
-	global->fps=videoIn->fps;
-	global->fps_num=videoIn->fps_num;
 	
 	gwidget = NULL;
 	global = NULL;
@@ -803,18 +820,8 @@ SndComp_changed (GtkComboBox * SoundComp, struct ALL_DATA *all_data)
 {
 	struct GLOBAL *global = all_data->global;
 	
-	/* 0-PCM (default) 1-MP2 */
-	switch (gtk_combo_box_get_active (SoundComp)) 
-	{
-		case 0:
-			global->Sound_Format  = PA_FOURCC;
-			break;
-		case 1:
-			global->Sound_Format = ISO_FORMAT_MPEG12;
-			break;
-		default:
-			global->Sound_Format  = PA_FOURCC;
-	}
+	/* 0-PCM  1-MP2 2-(lavc)... 3-(lavc)...*/
+	global->Sound_Format = get_aud4cc(gtk_combo_box_get_active (SoundComp)); 
 	
 	global = NULL;
 }
@@ -969,14 +976,7 @@ capture_image (GtkButton *ImageButt, struct ALL_DATA *all_data)
 	} 
 	else 
 	{
-		if(global->imgFormat == 3) 
-		{ /*raw frame*/
-			videoIn->cap_raw = 1;
-		} 
-		else 
-		{
-			videoIn->capImage = TRUE;
-		}
+		videoIn->capImage = TRUE;
 	}
 	
 	gwidget = NULL;
@@ -993,6 +993,13 @@ capture_vid (GtkToggleButton *VidButt, struct ALL_DATA *all_data)
 	struct paRecordData *pdata = all_data->pdata;
 	struct GLOBAL *global = all_data->global;
 	struct vdIn *videoIn = all_data->videoIn;
+
+	g_mutex_lock(videoIn->mutex);
+		gboolean capVid = videoIn->capVid;
+	g_mutex_unlock(videoIn->mutex);
+	/*disable signals for this callback*/
+	g_signal_handlers_block_by_func(GTK_TOGGLE_BUTTON(gwidget->CapVidButt), G_CALLBACK (capture_vid), all_data);
+	/*widgets are enable/disable in create_video.c*/
 	
 	const char *fileEntr = gtk_entry_get_text(GTK_ENTRY(gwidget->VidFNameEntry));
 	if(g_strcmp0(fileEntr,global->vidFPath[0])!=0) 
@@ -1002,26 +1009,47 @@ capture_vid (GtkToggleButton *VidButt, struct ALL_DATA *all_data)
 		global->vidFPath=splitPath((char *)fileEntr, global->vidFPath);
 		gtk_entry_set_text(GTK_ENTRY(gwidget->VidFNameEntry),"");
 		gtk_entry_set_text(GTK_ENTRY(gwidget->VidFNameEntry),global->vidFPath[0]);
+		/*get the file type*/
+		global->VidFormat = check_video_type(global->vidFPath[0]);
+		/*set the file type*/
+		gtk_combo_box_set_active(GTK_COMBO_BOX(gwidget->VidFormat),global->VidFormat);
 	}
 	
 	gboolean state = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(gwidget->CapVidButt));
 	if(global->debug) g_printf("Cap Video toggled: %d\n", state);
 	
-	if(videoIn->capVid || !state) 
+	if(capVid || !state) 
 	{	/****************** Stop Video ************************/
-		closeVideoFile(all_data);
+		capVid = FALSE;
+		g_mutex_lock(videoIn->mutex);
+			videoIn->capVid = capVid;
+		g_mutex_unlock(videoIn->mutex);
+		g_mutex_lock(pdata->mutex);
+			pdata->capVid = capVid;
+		g_mutex_unlock(pdata->mutex);
+		/*join IO thread*/
+		if (global->debug) g_printf("Shuting Down IO Thread\n");
+		g_thread_join( all_data->IO_thread );
+		if (global->debug) g_printf("IO Thread finished\n");
+		
+		if(global->debug) g_printf("enabling controls\n");
+		/*enabling sound and video compression controls*/
+		set_sensitive_vid_contrls(TRUE, global->Sound_enable, gwidget);
+		
 		if(!(state))
 		{
 			gtk_button_set_label(GTK_BUTTON(gwidget->CapVidButt),_("Cap. Video"));
-			gtk_widget_show (gwidget->VidButton_Img);
+			//gtk_widget_show (gwidget->VidButton_Img);
 		}
+		if(global->disk_timer_id) g_source_remove(global->disk_timer_id);
+		global->disk_timer_id = 0;
 	} 
-	else if(!(videoIn->capVid) /*&& state*/)
+	else if(!(capVid) /*&& state*/)
 	{	/******************** Start Video *********************/
 		global->vidFPath=splitPath((char *)fileEntr, global->vidFPath);
 		g_snprintf(global->vidinc_str,24,_("File num:%d"),global->vid_inc);
 		gtk_label_set_text(GTK_LABEL(gwidget->VidIncLabel), global->vidinc_str);
-	
+				
 		if (global->vid_inc>0) 
 		{
 			videoIn->VidFName = incFilename(videoIn->VidFName,
@@ -1034,19 +1062,52 @@ capture_vid (GtkToggleButton *VidButt, struct ALL_DATA *all_data)
 		{
 			videoIn->VidFName = joinPath(videoIn->VidFName, global->vidFPath);
 		}
-		
-		if(initVideoFile(all_data)<0)
+
+		/* check if enough free space is available on disk*/
+		if(!DiskSupervisor(all_data))
 		{
 			g_printf("Cap Video failed\n");
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(gwidget->CapVidButt), FALSE);
 			state = FALSE;
 		}
+		else
+		{
+			/*start disk check timed callback (every 10 sec)*/
+			if (!global->disk_timer_id)
+				global->disk_timer_id=g_timeout_add(10*1000, FreeDiskCheck_timer, all_data);
+			
+			/*disabling sound and video compression controls*/
+			set_sensitive_vid_contrls(FALSE, global->Sound_enable, gwidget);
+			
+			GError *err1 = NULL;
+			/*start IO thread*/
+			if( (all_data->IO_thread = g_thread_create_full((GThreadFunc) IO_loop, 
+				(void *) all_data,       //data - argument supplied to thread
+				global->stack_size,       //stack size
+				TRUE,                     //joinable
+				FALSE,                    //bound
+				G_THREAD_PRIORITY_NORMAL, //priority - no priority for threads in GNU-Linux
+				&err1)                    //error
+			) == NULL)
+			{
+				g_printerr("Thread create failed: %s!!\n", err1->message );
+				g_error_free ( err1 ) ;
+				state = FALSE;
+			}
+		}
+		
 		if(state)
 		{
 			gtk_button_set_label(GTK_BUTTON(gwidget->CapVidButt),_("Stop Video"));
-			gtk_widget_show (gwidget->VidButton_Img);
+			//gtk_widget_show (gwidget->VidButton_Img);
+		}
+		else
+		{
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(gwidget->CapVidButt), FALSE);
 		}
 	}
+
+	/*enable signals for this callback*/
+	g_signal_handlers_unblock_by_func(GTK_TOGGLE_BUTTON(gwidget->CapVidButt), G_CALLBACK (capture_vid), all_data);
 	
 	gwidget = NULL;
 	pdata = NULL;
@@ -1055,6 +1116,14 @@ capture_vid (GtkToggleButton *VidButt, struct ALL_DATA *all_data)
 }
 
 /*--------------------- buttons callbacks ------------------*/
+void
+DefaultsButton_clicked (GtkButton * DefaultsButton, struct ALL_DATA *all_data)
+{
+    struct VidState *s = all_data->s;
+    struct vdIn *videoIn = all_data->videoIn;
+
+    set_default_values(videoIn->fd, s->control_list, s->num_controls, all_data);
+}
 
 void
 ProfileButton_clicked (GtkButton * ProfileButton, struct ALL_DATA *all_data)
@@ -1099,9 +1168,9 @@ ProfileButton_clicked (GtkButton * ProfileButton, struct ALL_DATA *all_data)
 		global->profile_FPath=splitPath(filename,global->profile_FPath);
 		
 		if(*save)
-			SaveControls(s, global, videoIn);
+			SaveControls(all_data);
 		else
-			LoadControls(s,global);
+			LoadControls(all_data);
 	}
 	gtk_widget_destroy (gwidget->FileDialog);
 	gwidget = NULL;
@@ -1133,17 +1202,18 @@ split_avi(void *data)
 	//gtk_toggle_button_toggled (GTK_TOGGLE_BUTTON(gwidget->CapAVIButt));
 	gdk_flush ();
 	gdk_threads_leave();
-	int stall = wait_ms(&(videoIn->VidCapStop), TRUE, 10, 200);
+
+	/*FIXME: should join the caller thread instead (will it work?)*/
+	int stall = wait_ms(&(videoIn->IOfinished), TRUE, videoIn->mutex, 10, 200);
 	if( !(stall > 0) )
 	{
-		g_printerr("video capture stall on exit(%d) - timeout\n",
-			videoIn->VidCapStop);
+		g_printerr("IO thread stalled (%d) - timeout\n",
+			videoIn->IOfinished);
 	}
 	/*starts avi capture*/
 	gdk_threads_enter();
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(gwidget->CapVidButt), TRUE);
-	//gtk_toggle_button_toggled (GTK_TOGGLE_BUTTON(gwidget->CapAVIButt));
-	global->VidButtPress = FALSE;
+
 	gdk_flush ();
 	gdk_threads_leave();
 	/*thread as finished*/
