@@ -45,7 +45,7 @@
 
 /*-------------------------- clean up and shut down --------------------------*/
 
-static void
+void
 clean_struct (struct ALL_DATA *all_data) 
 {
 	struct GWIDGET *gwidget = all_data->gwidget;
@@ -55,10 +55,12 @@ clean_struct (struct ALL_DATA *all_data)
 	struct focusData *AFdata = all_data->AFdata;
 	struct vdIn *videoIn = all_data->videoIn;
 	struct VideoFormatData *videoF = all_data->videoF;
-
-	if(!(global->control_only))
+	
+	gboolean control_only = (global->control_only || global->add_ctrls);
+	if((!control_only) && (pdata != NULL))
 	{
 		/*destroy mutex for sound buffers*/
+		if (global->debug) g_printf("free audio mutex\n");
 		g_mutex_free( pdata->mutex );
 
 		g_free(pdata);
@@ -68,19 +70,18 @@ clean_struct (struct ALL_DATA *all_data)
 		videoF=NULL;
 	}
 	
-	if(videoIn) close_v4l2(videoIn);
+	if(videoIn) close_v4l2(videoIn, control_only);
 	videoIn=NULL;
-
 	if (global->debug) g_printf("closed v4l2 strutures\n");
 
 	
 
-	if (s->control) 
+	if (s->control_list) 
 	{
-		input_free_controls (s);
+		free_control_list (s->control_list);
+		s->control_list = NULL;
 		g_printf("free controls\n");
 	}
-
 	g_free(s);
 	s = NULL;
 	all_data->s = NULL;
@@ -114,16 +115,17 @@ shutd (gint restart, struct ALL_DATA *all_data)
 	struct GLOBAL *global = all_data->global;
 	struct vdIn *videoIn = all_data->videoIn;
 
-	gboolean control_only = global->control_only;
-
-	if (global->debug) g_printf("Shuting Down Thread\n");
-	if(videoIn->signalquit > 0) videoIn->signalquit=0;
-	if (global->debug) g_printf("waiting for thread to finish\n");
+	gboolean control_only = (global->control_only || global->add_ctrls);
 	
 	/* wait for the main loop (video) thread */
 	if(!(control_only))
 	{ 
+		if (global->debug) g_printf("Shuting Down Thread\n");
+		g_mutex_lock(videoIn->mutex);
+			videoIn->signalquit=TRUE;
+		g_mutex_unlock(videoIn->mutex);
 		g_thread_join( video_thread );
+		if (global->debug) g_printf("Video Thread finished\n");
 	}
 
 	/* destroys fps timer*/
