@@ -56,6 +56,37 @@ yuyv2rgb (BYTE *pyuv, BYTE *prgb, int width, int height)
 	}
 }
 
+/* used for rgb video (fourcc="RGB ")           */
+/* lines are on correct order                   */
+void 
+yuyv2bgr1 (BYTE *pyuv, BYTE *pbgr, int width, int height)
+{
+
+	int l=0;
+	int SizeYUV=height * width * 2; /* 2 bytes per pixel*/
+	for(l=0;l<SizeYUV;l=l+4) 
+	{	/*iterate every 4 bytes*/
+		/* standart: b = y0 + 1.772 (u-128) */
+		/* logitech: b = y0 + 1.732446 (u-128) */
+		*pbgr++=CLIP(pyuv[l] + 1.772 *( pyuv[l+1]-128));
+		/* standart: g = y0 - 0.34414 (u-128) - 0.71414 (v-128)*/
+		/* logitech: g = y0 - 0.337633 (u-128)- 0.698001 (v-128)*/
+		*pbgr++=CLIP(pyuv[l] - 0.34414 * (pyuv[l+1]-128) -0.71414*(pyuv[l+3]-128));
+		/* standart: r = y0 + 1.402 (v-128) */
+		/* logitech: r = y0 + 1.370705 (v-128) */
+		*pbgr++=CLIP(pyuv[l] + 1.402 * (pyuv[l+3]-128));
+		/* standart: b1 = y1 + 1.772 (u-128) */
+		/* logitech: b1 = y1 + 1.732446 (u-128) */
+		*pbgr++=CLIP(pyuv[l+2] + 1.772*(pyuv[l+1]-128));
+		/* standart: g1 = y1 - 0.34414 (u-128) - 0.71414 (v-128)*/
+		/* logitech: g1 = y1 - 0.337633 (u-128)- 0.698001 (v-128)*/
+		*pbgr++=CLIP(pyuv[l+2] - 0.34414 * (pyuv[l+1]-128) -0.71414 * (pyuv[l+3]-128));
+		/* standart: r1 =y1 + 1.402 (v-128) */
+		/* logitech: r1 = y1 + 1.370705 (v-128) */
+		*pbgr++=CLIP(pyuv[l+2] + 1.402 * (pyuv[l+3]-128));
+	}
+}
+
 /* yuv (YUYV) to bgr with lines upsidedown */
 /* used for bitmap files (DIB24)           */
 void 
@@ -215,7 +246,7 @@ void yvyu_to_yuyv (BYTE *framebuffer, BYTE *tmpbuffer, int width, int height)
 *      width: picture width
 *      height: picture height
 */
-int yuv420_to_yuyv (BYTE *framebuffer, BYTE *tmpbuffer, int width, int height) 
+void yuv420_to_yuyv (BYTE *framebuffer, BYTE *tmpbuffer, int width, int height) 
 {
 	BYTE *py;
 	BYTE *pu;
@@ -223,6 +254,12 @@ int yuv420_to_yuyv (BYTE *framebuffer, BYTE *tmpbuffer, int width, int height)
 	
 	int linesize = width * 2;
 	int uvlinesize = width / 2;
+	int offset=0;
+	int offset1=0;
+	int offsety=0;
+	int offsety1=0;
+	int offsetuv=0;
+	
 	py=tmpbuffer;
 	pu=py+(width*height);
 	pv=pu+(width*height/4);
@@ -238,31 +275,37 @@ int yuv420_to_yuyv (BYTE *framebuffer, BYTE *tmpbuffer, int width, int height)
 	{
 		wy=0;
 		wuv=0;
+		offset = h * linesize;
+		offset1 = (h + 1) * linesize;
+		offsety = h * width;
+		offsety1 = (h + 1) * width;
+		offsetuv = huv * uvlinesize;
+		
 		for(w=0;w<linesize;w+=4) 
 		{
 			/*y00*/
-			framebuffer[h * linesize + w] = py[h * width + wy];
+			framebuffer[w + offset] = py[wy + offsety];
 			/*u0*/
-			framebuffer[h * linesize + (w + 1)] = pu[huv * uvlinesize + wuv];
+			framebuffer[(w + 1) + offset] = pu[wuv + offsetuv];
 			/*y01*/
-			framebuffer[h * linesize + (w + 2)] = py[h * width + (wy + 1)];
+			framebuffer[(w + 2) + offset] = py[(wy + 1) + offsety];
 			/*v0*/
-			framebuffer[h * linesize + (w + 3)] = pv[huv * uvlinesize + wuv];
+			framebuffer[(w + 3) + offset] = pv[wuv + offsetuv];
 			
 			/*y10*/
-			framebuffer[(h + 1) * linesize + w] = py[(h + 1) * width + wy];
+			framebuffer[w + offset1] = py[wy + offsety1];
 			/*u0*/
-			framebuffer[(h + 1) * linesize + ( w + 1)] = pu[huv * uvlinesize + wuv];
+			framebuffer[(w + 1) + offset1] = pu[wuv + offsetuv];
 			/*y11*/
-			framebuffer[(h + 1) * linesize + (w + 2)] = py[(h + 1) * width + (wy + 1)];
+			framebuffer[(w + 2) + offset1] = py[(wy + 1) + offsety1];
 			/*v0*/
-			framebuffer[(h + 1) * linesize + (w + 3)] = pv[huv * uvlinesize + wuv];
+			framebuffer[(w + 3) + offset1] = pv[wuv + offsetuv];
+			
 			wuv++;
 			wy+=2;
 		}
 		huv++;
 	}
-	return (0);
 }
 
 /*convert yvu 420 planar (yv12) to yuv 422
@@ -272,7 +315,7 @@ int yuv420_to_yuyv (BYTE *framebuffer, BYTE *tmpbuffer, int width, int height)
 *      width: picture width
 *      height: picture height
 */
-int yvu420_to_yuyv (BYTE *framebuffer, BYTE *tmpbuffer, int width, int height) 
+void yvu420_to_yuyv (BYTE *framebuffer, BYTE *tmpbuffer, int width, int height) 
 {
 	BYTE *py;
 	BYTE *pv;
@@ -280,6 +323,12 @@ int yvu420_to_yuyv (BYTE *framebuffer, BYTE *tmpbuffer, int width, int height)
 	
 	int linesize = width * 2;
 	int uvlinesize = width / 2;
+	int offset=0;
+	int offset1=0;
+	int offsety=0;
+	int offsety1=0;
+	int offsetuv=0;
+	
 	py=tmpbuffer;
 	pv=py+(width*height);
 	pu=pv+((width*height)/4);
@@ -295,31 +344,451 @@ int yvu420_to_yuyv (BYTE *framebuffer, BYTE *tmpbuffer, int width, int height)
 	{
 		wy=0;
 		wuv=0;
+		offset = h * linesize;
+		offset1 = (h + 1) * linesize;
+		offsety = h * width;
+		offsety1 = (h + 1) * width;
+		offsetuv = huv * uvlinesize;
+		
 		for(w=0;w<linesize;w+=4) 
 		{
 			/*y00*/
-			framebuffer[h * linesize + w] = py[h * width + wy];
+			framebuffer[w + offset] = py[wy + offsety];
 			/*u0*/
-			framebuffer[h * linesize + (w + 1)] = pu[huv * uvlinesize + wuv];
+			framebuffer[(w + 1) + offset] = pu[wuv + offsetuv];
 			/*y01*/
-			framebuffer[h * linesize + (w + 2)] = py[h * width + (wy + 1)];
+			framebuffer[(w + 2) + offset] = py[(wy + 1) + offsety];
 			/*v0*/
-			framebuffer[h * linesize + (w + 3)] = pv[huv * uvlinesize + wuv];
+			framebuffer[(w + 3) + offset] = pv[wuv + offsetuv];
 			
 			/*y10*/
-			framebuffer[(h + 1) * linesize + w] = py[(h + 1) * width + wy];
+			framebuffer[w + offset1] = py[wy + offsety1];
 			/*u0*/
-			framebuffer[(h + 1) * linesize + (w + 1)] = pu[huv * uvlinesize + wuv];
+			framebuffer[(w + 1) + offset1] = pu[wuv + offsetuv];
 			/*y11*/
-			framebuffer[(h + 1) * linesize + (w + 2)] = py[(h + 1) * width + (wy + 1)];
+			framebuffer[(w + 2) + offset1] = py[(wy + 1) + offsety1];
 			/*v0*/
-			framebuffer[(h + 1) * linesize + (w + 3)] = pv[huv * uvlinesize + wuv];
+			framebuffer[(w + 3) + offset1] = pv[wuv + offsetuv];
+			
 			wuv++;
 			wy+=2;
 		}
 		huv++;
 	}
-	return (0);
+}
+
+/*convert yuv 420 planar (uv interleaved) (nv12) to yuv 422
+* args: 
+*      framebuffer: pointer to frame buffer (yuyv)
+*      tmpbuffer: pointer to temp buffer containing yuv420 (nv12) planar data frame
+*      width: picture width
+*      height: picture height
+*/
+void nv12_to_yuyv (BYTE *framebuffer, BYTE *tmpbuffer, int width, int height) 
+{
+	BYTE *py;
+	BYTE *puv;
+	
+	int linesize = width * 2;
+	int offset=0;
+	int offset1=0;
+	int offsety=0;
+	int offsety1=0;
+	int offsetuv=0;
+	
+	py=tmpbuffer;
+	puv=py+(width*height);
+	
+	int h=0;
+	int w=0;
+	
+	int wy=0;
+	int huv=0;
+	int wuv=0;
+	
+	for(h=0;h<height;h+=2) 
+	{
+		wy=0;
+		wuv=0;
+		offset = h * linesize;
+		offset1 = (h+1) * linesize;
+		offsety = h * width;
+		offsety1 = (h+1) * width;
+		offsetuv = huv * width;
+		for(w=0;w<linesize;w+=4) 
+		{
+			/*y00*/
+			framebuffer[w + offset] = py[wy + offsety];
+			/*u0*/
+			framebuffer[(w + 1) + offset] = puv[wuv + offsetuv];
+			/*y01*/
+			framebuffer[(w + 2) + offset] = py[(wy + 1) + offsety];
+			/*v0*/
+			framebuffer[(w + 3) + offset] = puv[(wuv + 1) + offsetuv];
+			
+			/*y10*/
+			framebuffer[w + offset1] = py[wy + offsety1];
+			/*u0*/
+			framebuffer[(w + 1) + offset1] = puv[wuv + offsetuv];
+			/*y11*/
+			framebuffer[(w + 2) + offset1] = py[(wy + 1) + offsety1];
+			/*v0*/
+			framebuffer[(w + 3) + offset1] = puv[(wuv + 1) + offsetuv];
+			
+			wuv+=2;
+			wy+=2;
+		}
+		huv++;
+	}
+}
+
+/*convert yuv 420 planar (vu interleaved) (nv21) to yuv 422
+* args: 
+*      framebuffer: pointer to frame buffer (yuyv)
+*      tmpbuffer: pointer to temp buffer containing yuv420 (nv21) planar data frame
+*      width: picture width
+*      height: picture height
+*/
+void nv21_to_yuyv (BYTE *framebuffer, BYTE *tmpbuffer, int width, int height) 
+{
+	BYTE *py;
+	BYTE *puv;
+	
+	int linesize = width * 2;
+	int offset=0;
+	int offset1=0;
+	int offsety=0;
+	int offsety1=0;
+	int offsetuv=0;
+	
+	py=tmpbuffer;
+	puv=py+(width*height);
+	
+	int h=0;
+	int w=0;
+	
+	int wy=0;
+	int huv=0;
+	int wuv=0;
+	
+	for(h=0;h<height;h+=2) 
+	{
+		wy=0;
+		wuv=0;
+		offset = h * linesize;
+		offset1 = (h+1) * linesize;
+		offsety = h * width;
+		offsety1 = (h+1) * width;
+		offsetuv = huv * width;
+		for(w=0;w<linesize;w+=4) 
+		{
+			/*y00*/
+			framebuffer[w + offset] = py[wy + offsety];
+			/*u0*/
+			framebuffer[(w + 1) + offset] = puv[(wuv + 1) + offsetuv];
+			/*y01*/
+			framebuffer[(w + 2) + offset] = py[(wy + 1) + offsety];
+			/*v0*/
+			framebuffer[(w + 3) + offset] = puv[wuv + offsetuv];
+			
+			/*y10*/
+			framebuffer[w + offset1] = py[wy + offsety1];
+			/*u0*/
+			framebuffer[(w + 1) + offset1] = puv[(wuv + 1) + offsetuv];
+			/*y11*/
+			framebuffer[(w + 2) + offset1] = py[(wy + 1) + offsety1];
+			/*v0*/
+			framebuffer[(w + 3) + offset1] = puv[wuv + offsetuv];
+			
+			wuv+=2;
+			wy+=2;
+		}
+		huv++;
+	}
+}
+
+/*convert yuv 422 planar (uv interleaved) (nv16) to yuv 422
+* args: 
+*      framebuffer: pointer to frame buffer (yuyv)
+*      tmpbuffer: pointer to temp buffer containing yuv422 (nv16) planar data frame
+*      width: picture width
+*      height: picture height
+*/
+void nv16_to_yuyv (BYTE *framebuffer, BYTE *tmpbuffer, int width, int height)
+{
+	BYTE *py;
+	BYTE *puv;
+	
+	int linesize = width * 2;
+	int offset=0;
+	int offsety=0;
+	int offsetuv=0;
+	
+	py=tmpbuffer;
+	puv=py+(width*height);
+	
+	int h=0;
+	int w=0;
+	
+	int wy=0;
+	int huv=0;
+	int wuv=0;
+	
+	for(h=0;h<height;h++) 
+	{
+		wy=0;
+		wuv=0;
+		offset = h * linesize;
+		offsety = h * width;
+		offsetuv = huv * width;
+		for(w=0;w<linesize;w+=4) 
+		{
+			/*y00*/
+			framebuffer[w + offset] = py[wy + offsety];
+			/*u0*/
+			framebuffer[(w + 1) + offset] = puv[wuv + offsetuv];
+			/*y01*/
+			framebuffer[(w + 2) + offset] = py[(wy + 1) + offsety];
+			/*v0*/
+			framebuffer[(w + 3) + offset] = puv[(wuv + 1) + offsetuv];
+			
+			wuv+=2;
+			wy+=2;
+		}
+		huv++;
+	}
+}
+
+/*convert yuv 422 planar (vu interleaved) (nv61) to yuv 422
+* args: 
+*      framebuffer: pointer to frame buffer (yuyv)
+*      tmpbuffer: pointer to temp buffer containing yuv422 (nv61) planar data frame
+*      width: picture width
+*      height: picture height
+*/
+void nv61_to_yuyv (BYTE *framebuffer, BYTE *tmpbuffer, int width, int height)
+{
+	BYTE *py;
+	BYTE *puv;
+	
+	int linesize = width * 2;
+	int offset=0;
+	int offsety=0;
+	int offsetuv=0;
+	
+	py=tmpbuffer;
+	puv=py+(width*height);
+	
+	int h=0;
+	int w=0;
+	
+	int wy=0;
+	int huv=0;
+	int wuv=0;
+	
+	for(h=0;h<height;h++) 
+	{
+		wy=0;
+		wuv=0;
+		offset = h * linesize;
+		offsety = h * width;
+		offsetuv = huv * width;
+		for(w=0;w<linesize;w+=4) 
+		{
+			/*y00*/
+			framebuffer[w + offset] = py[wy + offsety];
+			/*u0*/
+			framebuffer[(w + 1) + offset] = puv[(wuv + 1) + offsetuv];
+			/*y01*/
+			framebuffer[(w + 2) + offset] = py[(wy + 1) + offsety];
+			/*v0*/
+			framebuffer[(w + 3) + offset] = puv[wuv + offsetuv];
+			
+			wuv+=2;
+			wy+=2;
+		}
+		huv++;
+	}
+}
+
+/*convert yuv 411 packed (y41p) to yuv 422
+* args: 
+*      framebuffer: pointer to frame buffer (yuyv)
+*      tmpbuffer: pointer to temp buffer containing y41p data frame
+*      width: picture width
+*      height: picture height
+*/
+void y41p_to_yuyv (BYTE *framebuffer, BYTE *tmpbuffer, int width, int height)
+{
+	int h=0;
+	int w=0;
+	int linesize = width * 3 /2;
+	int offset = 0;
+	
+	for(h=0;h<height;h++)
+	{
+		offset = linesize * h;
+		for(w=0;w<linesize;w+=12)
+		{
+			*framebuffer++=tmpbuffer[w+1 + offset]; //Y0
+			*framebuffer++=tmpbuffer[w   + offset]; //U0
+			*framebuffer++=tmpbuffer[w+3 + offset]; //Y1
+			*framebuffer++=tmpbuffer[w+2 + offset]; //V0
+			*framebuffer++=tmpbuffer[w+5 + offset]; //Y2
+			*framebuffer++=tmpbuffer[w   + offset]; //U0
+			*framebuffer++=tmpbuffer[w+7 + offset]; //Y3
+			*framebuffer++=tmpbuffer[w+2 + offset]; //V0
+			*framebuffer++=tmpbuffer[w+8 + offset]; //Y4
+			*framebuffer++=tmpbuffer[w+4 + offset]; //U4
+			*framebuffer++=tmpbuffer[w+9 + offset]; //Y5
+			*framebuffer++=tmpbuffer[w+6 + offset]; //V4
+			*framebuffer++=tmpbuffer[w+10+ offset]; //Y6
+			*framebuffer++=tmpbuffer[w+4 + offset]; //U4
+			*framebuffer++=tmpbuffer[w+11+ offset]; //Y7
+			*framebuffer++=tmpbuffer[w+6 + offset]; //V4
+		}
+	}
+}
+
+/*convert yuv mono (grey) to yuv 422
+* args: 
+*      framebuffer: pointer to frame buffer (yuyv)
+*      tmpbuffer: pointer to temp buffer containing grey (y only) data frame
+*      width: picture width
+*      height: picture height
+*/
+void grey_to_yuyv (BYTE *framebuffer, BYTE *tmpbuffer, int width, int height)
+{
+	int h=0;
+	int w=0;
+	int offset = 0;
+	
+	for(h=0;h<height;h++)
+	{
+		offset = width * h;
+		for(w=0;w<width;w++)
+		{
+			*framebuffer++=tmpbuffer[w + offset]; //Y
+			*framebuffer++=0x80;                  //U or V
+		}
+	}
+}
+
+/*convert SPCA501 (s501) to yuv 422
+* s501  |Y0..width..Y0|U..width/2..U|Y1..width..Y1|V..width/2..V|
+* signed values (-128;+127) must be converted to unsigned (0; 255)
+* args: 
+*      framebuffer: pointer to frame buffer (yuyv)
+*      tmpbuffer: pointer to temp buffer containing s501 data frame
+*      width: picture width
+*      height: picture height
+*/
+void s501_to_yuyv(BYTE *framebuffer, BYTE *tmpbuffer, int width, int height)
+{
+	BYTE *U, *V, *Y0, *Y1;
+	BYTE *line2;
+	int h, w;
+
+	Y0 = tmpbuffer; /*fisrt line*/
+	for (h = 0; h < height/2; h++ ) 
+	{
+		line2 = framebuffer + width * 2;   /* next line          */
+		U = Y0 + width;
+		Y1 = U + width / 2;
+		V = Y1 + width;
+		for (w = width / 2; --w >= 0; ) 
+		{
+			*framebuffer++ = 0x80 + *Y0++;
+			*framebuffer++ = 0x80 + *U;
+			*framebuffer++ = 0x80 + *Y0++;
+			*framebuffer++ = 0x80 + *V;
+
+			*line2++ = 0x80 + *Y1++;
+			*line2++ = 0x80 + *U++;
+			*line2++ = 0x80 + *Y1++;
+			*line2++ = 0x80 + *V++;
+		}
+		Y0 += width * 2;                  /* next block of lines */
+		framebuffer = line2;
+	}
+}
+
+/*convert SPCA505 (s505) to yuv 422
+* s505  |Y0..width..Y0|Y1..width..Y1|U..width/2..U|V..width/2..V|
+* signed values (-128;+127) must be converted to unsigned (0; 255)
+* args: 
+*      framebuffer: pointer to frame buffer (yuyv)
+*      tmpbuffer: pointer to temp buffer containing s501 data frame
+*      width: picture width
+*      height: picture height
+*/
+void s505_to_yuyv(BYTE *framebuffer, BYTE *tmpbuffer, int width, int height)
+{
+	BYTE *U, *V, *Y0, *Y1;
+	BYTE *line2;
+	int h, w;
+
+	Y0 = tmpbuffer; /*fisrt line*/
+	for (h = 0; h < height/2; h++ ) 
+	{
+		line2 = framebuffer + width * 2;   /* next line          */
+		Y1 = Y0 + width;
+		U  = Y1 + width;
+		V  = U + width/2;
+		for (w = width / 2; --w >= 0; ) 
+		{
+			*framebuffer++ = 0x80 + *Y0++;
+			*framebuffer++ = 0x80 + *U;
+			*framebuffer++ = 0x80 + *Y0++;
+			*framebuffer++ = 0x80 + *V;
+
+			*line2++ = 0x80 + *Y1++;
+			*line2++ = 0x80 + *U++;
+			*line2++ = 0x80 + *Y1++;
+			*line2++ = 0x80 + *V++;
+		}
+		Y0 += width * 2;                  /* next block of lines */
+		framebuffer = line2;
+	}
+}
+
+/*convert SPCA508 (s508) to yuv 422
+* s508  |Y0..width..Y0|U..width/2..U|V..width/2..V|Y1..width..Y1|
+* signed values (-128;+127) must be converted to unsigned (0; 255)
+* args: 
+*      framebuffer: pointer to frame buffer (yuyv)
+*      tmpbuffer: pointer to temp buffer containing s501 data frame
+*      width: picture width
+*      height: picture height
+*/
+void s508_to_yuyv(BYTE *framebuffer, BYTE *tmpbuffer, int width, int height)
+{
+	BYTE *U, *V, *Y0, *Y1;
+	BYTE *line2;
+	int h, w;
+
+	Y0 = tmpbuffer; /*fisrt line*/
+	for (h = 0; h < height/2; h++ ) 
+	{
+		line2 = framebuffer + width * 2;   /* next line          */
+		U = Y0 + width;
+		V = U + width/2;
+		Y1= V + width/2;
+		for (w = width / 2; --w >= 0; ) 
+		{
+			*framebuffer++ = 0x80 + *Y0++;
+			*framebuffer++ = 0x80 + *U;
+			*framebuffer++ = 0x80 + *Y0++;
+			*framebuffer++ = 0x80 + *V;
+
+			*line2++ = 0x80 + *Y1++;
+			*line2++ = 0x80 + *U++;
+			*line2++ = 0x80 + *Y1++;
+			*line2++ = 0x80 + *V++;
+		}
+		Y0 += width * 2;                  /* next block of lines */
+		framebuffer = line2;
+	}
 }
 
 // raw bayer functions 
