@@ -35,6 +35,18 @@
 #define IO_MMAP 1
 #define IO_READ 2
 
+#define ODD(x) ((x%2)?TRUE:FALSE)
+
+/*next index of ring buffer with size elements*/
+#define NEXT_IND(ind,size) ind++;if(ind>=size) ind=0
+/*previous index of ring buffer with size elements*/
+//#define PREV_IND(ind,size) ind--;if(ind<0) ind=size-1
+
+#define VIDBUFF_SIZE 45    //number of video frames in the ring buffer
+#define AUDBUFF_SIZE 100   //number of audio mpeg frames in the ring buffer
+#define MPG_NUM_SAMP 1152  //number of samples in a audio MPEG frame 
+//#define MPG_NUM_FRAMES 2   //number of MPEG frames in a audio frame
+
 typedef uint64_t QWORD;
 typedef uint32_t DWORD;
 typedef uint16_t WORD;
@@ -56,6 +68,8 @@ typedef uint32_t   UINT32;
 typedef int64_t    INT64;
 typedef uint64_t   UINT64;
 
+typedef float SAMPLE;
+
 /* 0 is device default*/
 static const int stdSampleRates[] = 
 { 
@@ -97,7 +111,7 @@ static const int stdSampleRates[] =
 #define SDL_WAIT_TIME 30 /*SDL - Thread loop sleep time */
 
 /*clip value between 0 and 255*/
-#define CLIP(value) (unsigned char)(((value)>0xFF)?0xff:(((value)<0)?0:(value)))
+#define CLIP(value) (BYTE)(((value)>0xFF)?0xff:(((value)<0)?0:(value)))
 
 /*MAX macro - gets the bigger value*/
 #ifndef MAX
@@ -111,6 +125,7 @@ static const int stdSampleRates[] =
 #define YUV_NEGATE (1<<2)
 #define YUV_MONOCR (1<<3)
 #define YUV_PIECES (1<<4)
+#define YUV_PARTICLES (1<<5)
 
 /*Audio Effects*/
 #define SND_NOEF   (0)
