@@ -30,6 +30,7 @@
 int initGlobals (struct GLOBAL *global) 
 {
 	global->mutex = g_mutex_new();
+	global->IO_cond = g_cond_new();   /* Initialized video buffer semaphore */
 
 	global->debug = DEBUG;
 	
@@ -60,6 +61,8 @@ int initGlobals (struct GLOBAL *global)
 	global->WVcaption = g_new(char, 32);
 	
 	g_snprintf(global->WVcaption,10,"GUVCVIdeo");
+
+	global->videoBuff = NULL;
 	
 	global->stack_size=TSTACK;
 	
@@ -84,22 +87,25 @@ int initGlobals (struct GLOBAL *global)
 	global->Vidstarttime=0;
 	global->Vidstoptime=0;
 	global->framecount=0;
-	
+	global->w_ind=0;
+	global->r_ind=0;
+		
 	global->Sound_enable=TRUE; /*Enable Sound by Default*/
 	global->Sound_IndexDev=NULL;
 	global->Sound_SampRate=SAMPLE_RATE;
 	global->Sound_SampRateInd=0;
 	global->Sound_numInputDev=0;
-	global->Sound_Format=ISO_FORMAT_MPEG12; /*set MP2 by default*/
+	global->Sound_Format=WAVE_FORMAT_MPEG12; /*set MP2 by default*/
 	global->Sound_DefDev=0; 
 	global->Sound_UseDev=0;
 	global->Sound_NumChan=NUM_CHANNELS;
 	global->Sound_NumChanInd=0;
-	global->Sound_NumSec=NUM_SECONDS;
-	global->Sound_bitRate=160; /*160 Kbps = 20000 Bps*/
-	
+	global->Sound_delay=0;
+	//global->Sound_bitRate=160; /*160 Kbps = 20000 Bps*/
+	global->Sound_delay=0;     /*sound delay in nanosec*/
 	global->FpsCount=0;
 
+	global->disk_timer_id=0;
 	global->timer_id=0;
 	global->image_timer_id=0;
 	global->image_timer=0;
@@ -129,14 +135,14 @@ int initGlobals (struct GLOBAL *global)
 	global->Frame_Flags = YUV_NOFILT;
 
 	global->exit_on_close = FALSE;
+	global->skip_n=0;
 	global->jpeg=NULL;
-	global->jpeg_size = 0;
 	/* reset with videoIn parameters */
-	global->jpeg_bufsize = 0;
 	global->autofocus = FALSE;
 	global->AFcontrol = FALSE;
 	global->VidButtPress = FALSE;
-	
+	global->change_res = FALSE;
+	global->add_ctrls = FALSE;
 	return (0);
 }
 
@@ -160,12 +166,13 @@ int closeGlobals(struct GLOBAL *global)
 	g_free(global->mode);
 	g_free(global->Sound_IndexDev);
 	g_mutex_free( global->mutex );
+	g_cond_free( global->IO_cond);
 	
 	global->videodevice=NULL;
 	global->confPath=NULL;
 	global->vidfile=NULL;
 	global->mode=NULL;
-	g_free(global->jpeg);
+	if(global->jpeg) g_free(global->jpeg);
 	global->jpeg=NULL;
 	g_free(global);
 	global=NULL;
