@@ -35,10 +35,22 @@ typedef struct _sndDev
 	//PaTime Llatency;
 } sndDev;
 
+typedef struct _VidBuff
+{
+	gboolean used;
+	QWORD time_stamp;
+	BYTE *frame;
+	int bytes_used;
+} VidBuff;
+
 /*global variables used in guvcview*/
 struct GLOBAL 
 {
 	GMutex *mutex;         //global structure mutex
+	GCond   *IO_cond;      //IO thread semaphore
+	
+	VidBuff *videoBuff;    //video Buffer
+	
 	char *videodevice;     // video device (def. /dev/video0)
 	char *confPath;        //configuration file path
 	char *vidfile;         //video filename passed through argument options with -n
@@ -57,15 +69,14 @@ struct GLOBAL
 	DWORD snd_begintime;   //begin time for audio capture
 	DWORD currtime;
 	DWORD lasttime;
-	QWORD Vidstarttime;    //video start time
-	QWORD Vidstoptime;     //video stop time
+	UINT64 Vidstarttime;    //video start time
+	UINT64 Vidstoptime;     //video stop time
 	QWORD v_ts;            //video time stamp
 	QWORD a_ts;            //audio time stamp
 	DWORD vid_inc;         //video name increment
 	DWORD framecount;      //video frame count
 	DWORD frmCount;        //frame count for fps display calc
 	DWORD image_inc;       //image name increment
-	DWORD jpeg_bufsize;    // width*height/2 
 
 	int stack_size;        //thread stack size
 	int vid_sleep;         //video thread sleep time (0 by default)
@@ -73,7 +84,7 @@ struct GLOBAL
 	int Capture_time;      //video capture time passed through argument options with -t 
 	int imgFormat;         //image format: 0-"jpg", 1-"png", 2-"bmp"
 	int VidCodec;          //0-"MJPG"  1-"YUY2" 2-"DIB "(rgb32) 3-....
-	int VidFormat;         //0-AVI 1-MPEG 2-FLV 3- MKV ....
+	int VidFormat;         //0-AVI 1-MKV ....
 	int Sound_API;         //audio API: 0-PORTAUDIO 1-PULSEAUDIO
 	int Sound_SampRate;    //audio sample rate
 	int Sound_SampRateInd; //audio sample rate combo index
@@ -82,14 +93,15 @@ struct GLOBAL
 	int Sound_UseDev;      //audio used device index
 	int Sound_NumChan;     //audio number of channels
 	int Sound_NumChanInd;  //audio number of channels combo index
-	int Sound_NumSec;      //number of audio seconds to save into video on iteration (def. 1)
-	int Sound_Format;      //audio format (mpeg2 or 16 bit PCM)
-	int Sound_bitRate;     //bit rate for mpeg audio compression
+	WORD Sound_Format;     //audio codec - fourcc (avilib.h)
+	//int Sound_bitRate;     //bit rate for mpeg audio compression
+	UINT64 Sound_delay;    //sound delay in nanosec
 	int PanStep;           //step angle for Pan
 	int TiltStep;          //step angle for Tilt
 	int FpsCount;          //frames counter for fps calc
 	int timer_id;          //fps count timer
 	int image_timer_id;    //auto image capture timer
+	int disk_timer_id;     //timer id for disk check (free space)
 	int image_timer;       //auto image capture time
 	int image_npics;       //number of captures
 	int fps;               //fps denominator
@@ -104,7 +116,9 @@ struct GLOBAL
 	int spinbehave;        //spin: 0-non editable 1-editable
 	int format;            //v4l2 pixel format
 	int Frame_Flags;       //frame filter flags
-	int jpeg_size;         //jpeg buffer size
+	int skip_n;            //initial frames to skip
+	int w_ind;             //write frame index
+	int r_ind;             //read  frame index
 
 	float DispFps;         //fps value
 
@@ -124,6 +138,8 @@ struct GLOBAL
 	gboolean debug;        //debug mode flag (--verbose)
 	gboolean VidButtPress;
 	gboolean control_only; //if set don't stream video (enables image control in other apps e.g. ekiga, skype, mplayer)
+	gboolean change_res;   //flag for reseting resolution
+	gboolean add_ctrls;    //flag for exiting after adding extension controls
 };
 
 /*----------------------------- prototypes ------------------------------------*/
