@@ -48,7 +48,6 @@
 #include "video.h"
 #include "vcodecs.h"
 #include "create_video.h"
-#include "mp2.h"
 #include "profile.h"
 #include "callbacks.h"
 #include "close.h"
@@ -73,9 +72,6 @@ struct GWIDGET *gwidget = NULL;
 /*thread definitions*/
 GThread *video_thread = NULL;
 
-/* parameters passed when restarting*/
-//gchar *EXEC_CALL;
-
 /*--------------------------------- MAIN -------------------------------------*/
 int main(int argc, char *argv[])
 {
@@ -90,7 +86,6 @@ int main(int argc, char *argv[])
 	{
 		g_thread_init(NULL);
 		gdk_threads_init ();
-		//printf("g_thread supported\n");
 	}
 	else
 	{
@@ -119,7 +114,9 @@ int main(int argc, char *argv[])
 	/*------------------------- reads configuration file ---------------------*/
 	readConf(global);
 	
-	control_only = global->control_only; //sets local control_only flag
+	//sets local control_only flag - prevents several initializations/allocations
+	control_only = (global->control_only || global->add_ctrls) ;
+	
 	/*---------------------------------- Allocations -------------------------*/
 	
 	gwidget = g_new0(struct GWIDGET, 1);
@@ -133,15 +130,18 @@ int main(int argc, char *argv[])
 	GtkWidget *quitButton;
 	GtkWidget *SProfileButton;
 	GtkWidget *LProfileButton;
+    GtkWidget *DefaultsButton;
 	GtkWidget *Tab1;
 	GtkWidget *Tab1Label;
 	GtkWidget *Tab1Icon;
 	GtkWidget *ImgButton_Img;
 	GtkWidget *SButton_Img;
 	GtkWidget *LButton_Img;
+	GtkWidget *DButton_Img;
 	GtkWidget *QButton_Img;
 	GtkWidget *HButtonBox;
-
+	
+	
 	s = g_new0(struct VidState, 1);
 
 	if(!control_only) /*control_only exclusion (video and Audio) */
@@ -174,7 +174,14 @@ int main(int argc, char *argv[])
 	/*---------------------------- GTK init ----------------------------------*/
 
 	gtk_init(&argc, &argv);
+	g_set_application_name(_("Guvcview Video Capture"));
+	g_setenv("PULSE_PROP_media.role", "video", TRUE); //needed for Pulse Audio
 
+	/* make sure the type is realized so that we can change the properties*/
+	g_type_class_unref (g_type_class_ref (GTK_TYPE_BUTTON));
+	/* make sure gtk-button-images property is set to true (defaults to false in karmic)*/
+	g_object_set (gtk_settings_get_default (), "gtk-button-images", TRUE, NULL);
+	
 	/* Create a main window */
 	gwidget->mainwin = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 	gtk_window_set_title (GTK_WINDOW (gwidget->mainwin), _("GUVCViewer Controls"));
@@ -198,7 +205,7 @@ int main(int argc, char *argv[])
 	global->format = get_PixFormat(global->mode);
 	if(global->debug) g_printf("%s: setting format to %i\n", global->mode, global->format);
 
-	if ( ( ret=init_videoIn (videoIn, global) ) < 0)
+	if ( ( ret=init_videoIn (videoIn, global) ) != 0)
 	{
 		g_printerr("Init video returned %i\n",ret);
 		switch (ret) 
@@ -208,6 +215,20 @@ int main(int argc, char *argv[])
 					N_("Please make sure the camera is connected\nand that the correct driver is installed."),
 					&all_data);
 				break;
+			
+			case VDIN_DYNCTRL_OK: //uvc extension controls OK, give warning and shutdown (called with --add_ctrls)
+				WARN_DIALOG (N_("Guvcview:\n\nUVC Extension controls"),
+					N_("Extension controls were added to the UVC driver"),
+					&all_data);
+				clean_struct(&all_data);
+				exit(0);
+				break;				
+
+			case VDIN_DYNCTRL_ERR: //uvc extension controls error - EACCES (needs root user)
+				ERR_DIALOG (N_("Guvcview error:\n\nUVC Extension controls"),
+					N_("An error ocurred while adding extension\ncontrols to the UVC driver\nMake sure you run guvcview as root (or sudo)."),
+					&all_data);
+				break;
 				
 			case VDIN_UNKNOWN_ERR: //unknown error (treat as invalid format)
 			case VDIN_FORMAT_ERR://invalid format
@@ -237,15 +258,52 @@ int main(int argc, char *argv[])
 				}
 				
 				//try again with new format
-				if (init_videoIn (videoIn, global) < 0)
+				ret = init_videoIn (videoIn, global);
+				
+				if ((ret == VDIN_QUERYBUF_ERR) && (global->cap_meth != videoIn->cap_meth))
+				{
+					//mmap not supported ? try again with read method
+					g_printerr("mmap failed trying read method...");
+					global->cap_meth = videoIn->cap_meth;
+					ret = init_videoIn (videoIn, global);
+					if (ret == VDIN_OK) 
+						g_printerr("OK\n");
+					else
+						g_printerr("FAILED\n");
+				} 
+				
+				if (ret < 0)
 				{
 					g_printerr("ERROR: Minimum Setup Failed.\n Exiting...\n");
 					ERR_DIALOG (N_("Guvcview error:\n\nUnable to start with minimum setup"),
 						N_("Please reconnect your camera."), 
 						&all_data);
 				}
+				
 				break;
-
+				
+			case VDIN_QUERYBUF_ERR:
+				if (global->cap_meth != videoIn->cap_meth)
+				{
+					//mmap not supported ? try again with read method
+					g_printerr("mmap failed trying read method...");
+					global->cap_meth = videoIn->cap_meth;
+					ret = init_videoIn (videoIn, global);
+					if (ret == VDIN_OK) 
+						g_printerr("OK\n");
+					else
+					{
+						g_printerr("FAILED\n");
+						//return to default method(mmap)
+						global->cap_meth = IO_MMAP;
+						g_printerr("ERROR: Minimum Setup Failed.\n Exiting...\n");
+						ERR_DIALOG (N_("Guvcview error:\n\nUnable to start with minimum setup"),
+							N_("Please reconnect your camera."), 
+							&all_data);
+					}
+				}
+				break;
+				
 			case VDIN_QUERYCAP_ERR:
 				ERR_DIALOG (N_("Guvcview error:\n\nCouldn't query device capabilities"),
 					N_("Make sure the device driver suports v4l2."),
@@ -267,12 +325,7 @@ int main(int argc, char *argv[])
 				break;
 		}
 	}
-	/*make sure global and videoIn are in sync after v4l2 initialization*/
-	global->width = videoIn->width;
-	global->height = videoIn->height;
-	global->fps = videoIn->fps;
-	global->fps_num = videoIn->fps_num;
-	global->format = videoIn->formatIn;
+	
 	videoIn->listFormats->current_format = get_FormatIndex(videoIn->listFormats, global->format);
 	if(videoIn->listFormats->current_format < 0) 
 	{
@@ -281,20 +334,14 @@ int main(int argc, char *argv[])
 			N_("Make sure your device driver is v4l2 compliant\nand that it is properly installed."),
 			&all_data);
 	}
-	/* Set jpeg encoder buffer size */
-	global->jpeg_bufsize=((videoIn->width)*(videoIn->height))>>1;
 	/*-----------------------------GTK widgets---------------------------------*/
 	/*----------------------- Image controls Tab ------------------------------*/
-	s->table = gtk_table_new (1, 3, FALSE);
-	gtk_table_set_row_spacings (GTK_TABLE (s->table), 4);
-	gtk_table_set_col_spacings (GTK_TABLE (s->table), 4);
-	gtk_container_set_border_width (GTK_CONTAINER (s->table), 2);
-	
-	s->control = NULL;
+	s->control_list = NULL;
 	/*-- draw the controls --*/
+	printf("drawing controls\n\n");
 	draw_controls(&all_data);
 	
-	if (global->lprofile > 0) LoadControls (s,global);
+	if (global->lprofile > 0) LoadControls (&all_data);
 	
 	gwidget->boxv = gtk_vpaned_new ();
 	gwidget->boxh = gtk_notebook_new();
@@ -367,6 +414,7 @@ int main(int argc, char *argv[])
 	quitButton=gtk_button_new_from_stock(GTK_STOCK_QUIT);
 	SProfileButton=gtk_button_new_from_stock(GTK_STOCK_SAVE);
 	LProfileButton=gtk_button_new_from_stock(GTK_STOCK_OPEN);
+    DefaultsButton=gtk_button_new_with_label(_("Defaults"));
 
 	gchar* icon1path = g_strconcat (PACKAGE_DATA_DIR,"/pixmaps/guvcview/guvcview.png",NULL);
 	if (g_file_test(icon1path,G_FILE_TEST_EXISTS))
@@ -389,7 +437,6 @@ int main(int argc, char *argv[])
 		if (global->vidfile) 
 		{	/*vid capture enabled from start*/
 			gwidget->CapVidButt=gtk_toggle_button_new_with_label (_("Stop Video"));
-		
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gwidget->CapVidButt), TRUE);
 		} 
 		else 
@@ -408,18 +455,17 @@ int main(int argc, char *argv[])
 		
 			gtk_button_set_image(GTK_BUTTON(gwidget->CapVidButt),gwidget->VidButton_Img);
 			gtk_button_set_image_position(GTK_BUTTON(gwidget->CapVidButt),GTK_POS_TOP);
-			gtk_widget_show (gwidget->VidButton_Img);
+			//gtk_widget_show (gwidget->VidButton_Img);
 		}
 		//else g_printf("couldn't load %s\n", pix1path);
 		gchar* pix2path = g_strconcat (PACKAGE_DATA_DIR,"/pixmaps/guvcview/camera.png",NULL);
 		if (g_file_test(pix2path,G_FILE_TEST_EXISTS)) 
 		{
 			ImgButton_Img = gtk_image_new_from_file (pix2path);
-			//gtk_image_set_pixel_size (GTK_IMAGE(ImgButton_Img), 64);
 		
 			gtk_button_set_image(GTK_BUTTON(gwidget->CapImageButt),ImgButton_Img);
 			gtk_button_set_image_position(GTK_BUTTON(gwidget->CapImageButt),GTK_POS_TOP);
-			gtk_widget_show (ImgButton_Img);
+			//gtk_widget_show (ImgButton_Img);
 		}
 		g_free(pix1path);
 		g_free(pix2path);
@@ -433,50 +479,62 @@ int main(int argc, char *argv[])
 			G_CALLBACK (capture_image), &all_data);
 		g_signal_connect (GTK_TOGGLE_BUTTON(gwidget->CapVidButt), "toggled",
 			G_CALLBACK (capture_vid), &all_data);
+        /*key events*/
+        gtk_widget_add_events (GTK_WIDGET (gwidget->mainwin), GDK_KEY_PRESS_MASK | GDK_KEY_RELEASE_MASK);
+        g_signal_connect (GTK_OBJECT(gwidget->mainwin), "key_press_event", G_CALLBACK (key_pressed), &all_data);
 	}/*end of control_only exclusion*/
 	
 	gchar* pix3path = g_strconcat (PACKAGE_DATA_DIR,"/pixmaps/guvcview/save.png",NULL);
 	if (g_file_test(pix3path,G_FILE_TEST_EXISTS)) 
 	{
 		SButton_Img = gtk_image_new_from_file (pix3path);
-		//gtk_image_set_pixel_size (GTK_IMAGE(SButton_Img), 64);
 		
 		gtk_button_set_image(GTK_BUTTON(SProfileButton),SButton_Img);
 		gtk_button_set_image_position(GTK_BUTTON(SProfileButton),GTK_POS_TOP);
-		gtk_widget_show (SButton_Img);
+		//gtk_widget_show (SButton_Img);
 	}
 	gchar* pix4path = g_strconcat (PACKAGE_DATA_DIR,"/pixmaps/guvcview/controls_folder.png",NULL);
 	if (g_file_test(pix4path,G_FILE_TEST_EXISTS)) 
 	{
 		LButton_Img = gtk_image_new_from_file (pix4path);
-		//gtk_image_set_pixel_size (GTK_IMAGE(LButton_Img), 64);
 		
 		gtk_button_set_image(GTK_BUTTON(LProfileButton),LButton_Img);
 		gtk_button_set_image_position(GTK_BUTTON(LProfileButton),GTK_POS_TOP);
-		gtk_widget_show (LButton_Img);
+		//gtk_widget_show (LButton_Img);
 	}
-	gchar* pix5path = g_strconcat (PACKAGE_DATA_DIR,"/pixmaps/guvcview/close.png",NULL);
+    gchar* pix5path = g_strconcat (PACKAGE_DATA_DIR,"/pixmaps/guvcview/defaults.png",NULL);
 	if (g_file_test(pix5path,G_FILE_TEST_EXISTS)) 
 	{
-		QButton_Img = gtk_image_new_from_file (pix5path);
-		//gtk_image_set_pixel_size (GTK_IMAGE(QButton_Img), 64);
+		DButton_Img = gtk_image_new_from_file (pix5path);
+		
+		gtk_button_set_image(GTK_BUTTON(DefaultsButton),DButton_Img);
+		gtk_button_set_image_position(GTK_BUTTON(DefaultsButton),GTK_POS_TOP);
+		//gtk_widget_show (LButton_Img);
+	}
+	gchar* pix6path = g_strconcat (PACKAGE_DATA_DIR,"/pixmaps/guvcview/close.png",NULL);
+	if (g_file_test(pix6path,G_FILE_TEST_EXISTS)) 
+	{
+		QButton_Img = gtk_image_new_from_file (pix6path);
 		
 		gtk_button_set_image(GTK_BUTTON(quitButton),QButton_Img);
 		gtk_button_set_image_position(GTK_BUTTON(quitButton),GTK_POS_TOP);
-		gtk_widget_show (QButton_Img);
+		//gtk_widget_show (QButton_Img);
 	}
 
 	/*must free path strings*/
 	g_free(pix3path);
 	g_free(pix4path);
 	g_free(pix5path);
+    g_free(pix6path);
 
 	gtk_box_pack_start(GTK_BOX(HButtonBox),SProfileButton,TRUE,TRUE,2);
 	gtk_box_pack_start(GTK_BOX(HButtonBox),LProfileButton,TRUE,TRUE,2);
+    gtk_box_pack_start(GTK_BOX(HButtonBox),DefaultsButton,TRUE,TRUE,2);
 	gtk_box_pack_start(GTK_BOX(HButtonBox),quitButton,TRUE,TRUE,2);
 
 	gtk_widget_show_all (LProfileButton);
 	gtk_widget_show_all (SProfileButton);
+    gtk_widget_show_all (DefaultsButton);
 	gtk_widget_show_all (quitButton);
 
 	g_signal_connect (GTK_BUTTON(quitButton), "clicked",
@@ -490,6 +548,9 @@ int main(int argc, char *argv[])
 	g_object_set_data (G_OBJECT (LProfileButton), "profile_save", &(LProfile));
 	g_signal_connect (GTK_BUTTON(LProfileButton), "clicked",
 		G_CALLBACK (ProfileButton_clicked), &all_data);
+    
+    g_signal_connect (GTK_BUTTON(DefaultsButton), "clicked",
+		G_CALLBACK (DefaultsButton_clicked), &all_data);
 	
 	/*sets the pan position*/
 	if(global->boxvsize==0) 
@@ -502,6 +563,7 @@ int main(int argc, char *argv[])
 	{
 		/*------------------------- Video Tab ---------------------------------*/
 		video_tab (&all_data);
+		
 		/*-------------------------- Audio Tab --------------------------------*/
 		audio_tab (&all_data);
 	} /*end of control_only exclusion*/
@@ -513,15 +575,6 @@ int main(int argc, char *argv[])
 	
 	if (!control_only) /*control_only exclusion*/
 	{
-		/* if autofocus exists allocate data*/
-		if(global->AFcontrol) 
-		{
-			AFdata = g_new0(struct focusData, 1);
-			initFocusData(AFdata);
-		}
-
-		all_data.AFdata = AFdata;
-	
 		/*------------------ Creating the main loop (video) thread ---------------*/
 		GError *err1 = NULL;
 
@@ -554,13 +607,54 @@ int main(int argc, char *argv[])
 		if(global->vidfile) 
 		{
 			videoIn->VidFName = joinPath(videoIn->VidFName, global->vidFPath);
-			initVideoFile(&all_data);
-			if (global->Capture_time) 
+			
+			gboolean cap_ok = TRUE;
+			/* check if enough free space is available on disk*/
+			if(!DiskSupervisor(&all_data))
 			{
-				/*sets the timer function*/
-				g_timeout_add(global->Capture_time*1000,timer_callback,&all_data);
+				cap_ok = FALSE;
+			}
+			else
+			{
+				/*start disk check timed callback (every 10 sec)*/
+				if (!global->disk_timer_id)
+					global->disk_timer_id=g_timeout_add(10*1000, FreeDiskCheck_timer, &all_data);
+
+				GError *err1 = NULL;
+				/*start IO thread*/
+				if( (all_data.IO_thread = g_thread_create_full((GThreadFunc) IO_loop, 
+					(void *) &all_data,       //data - argument supplied to thread
+					global->stack_size,       //stack size
+					TRUE,                     //joinable
+					FALSE,                    //bound
+					G_THREAD_PRIORITY_NORMAL, //priority - no priority for threads in GNU-Linux
+					&err1)                    //error
+				) == NULL)
+				{
+					g_printerr("Thread create failed: %s!!\n", err1->message );
+					g_error_free ( err1 ) ;
+					cap_ok = FALSE;
+				}
+				else if (global->Capture_time) 
+				{
+					/*sets the timer function*/
+					g_timeout_add(global->Capture_time*1000,timer_callback,&all_data);
+				}
 			}
 			
+			if(!cap_ok)
+			{
+				g_printerr("ERROR: couldn't start video capture\n");
+				//g_signal_handlers_block_by_func(GTK_TOGGLE_BUTTON(gwidget->CapVidButt), G_CALLBACK (capture_vid), all_data);
+				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(gwidget->CapVidButt), FALSE);
+				gtk_button_set_label(GTK_BUTTON(gwidget->CapVidButt),_("Cap. Video"));
+				//g_signal_handlers_unblock_by_func(GTK_TOGGLE_BUTTON(gwidget->CapVidButt), G_CALLBACK (capture_vid), all_data);
+			}
+			else
+			{
+				/*disabling sound and video compression controls*/
+				set_sensitive_vid_contrls(FALSE, global->Sound_enable, gwidget);
+			}
 		}
 	
 		if (global->FpsCount>0)
@@ -586,7 +680,7 @@ int main(int argc, char *argv[])
 			g_printf("OK\n");
 	}
 	
-	//g_printf("Exit: OK\n");
+	g_printf("Closing GTK... OK\n");
 	return 0;
 }
 
