@@ -65,9 +65,12 @@ struct GWIDGET
 	GtkWidget *VidInc;
 	GtkWidget *VidFormat;
 	GtkWidget *Resolution;
+	GtkWidget *InpType;
+	GtkWidget *FrameRate;
 	GtkWidget *Devices;
 	GtkWidget *FileDialog;
 	GtkWidget *lavc_button;
+	GtkWidget *lavc_aud_button;
 	GtkWidget *jpeg_comp;
 	
 	gboolean vid_widget_state;
@@ -83,6 +86,7 @@ struct ALL_DATA
 	struct GWIDGET *gwidget;
 	struct VidState *s;
 	GThread *video_thread;
+	GThread *IO_thread;
 };
 
 #endif
