@@ -40,8 +40,12 @@ writeConf(struct GLOBAL *global, char *videodevice)
 {
 	int ret=0;
 	FILE *fp;
-
-	if ((fp = g_fopen(global->confPath,"w"))!=NULL) 
+	
+	// write to tmp file then rename after sucessfull fsync
+	// using fsync avois data loss on system crash 
+	// see: https://bugs.launchpad.net/ubuntu/+source/linux/+bug/317781/comments/54
+	char *tmpfile = g_strjoin (".", global->confPath, "tmp", NULL);
+	if ((fp = g_fopen(tmpfile,"w"))!=NULL) 
 	{
 		g_fprintf(fp,"# guvcview configuration file for %s\n\n",videodevice);
 		g_fprintf(fp,"# Thread stack size: default 128 pages of 64k = 8388608 bytes\n");
@@ -59,7 +63,7 @@ writeConf(struct GLOBAL *global, char *videodevice)
 		g_fprintf(fp,"vpane=%i\n",global->boxvsize);
 		g_fprintf(fp,"#spin button behavior: 0-non editable 1-editable\n");
 		g_fprintf(fp,"spinbehave=%i\n", global->spinbehave);
-		g_fprintf(fp,"# mode video format 'yuvy' 'yvyu' 'uyvy' 'yyuv' 'yu12' 'yv12' 'gbrg' 'grbg' 'bggr' 'rggb' 'rgb3' 'bgr3' 'jpeg' 'mjpg' 'mjpg'(default)\n");
+		g_fprintf(fp,"# mode video format 'yuvy' 'yvyu' 'uyvy' 'yyuv' 'yu12' 'yv12' 'nv12' 'nv21' 'nv16' 'nv61' 'y41p' 'grey' 's501' 's505' 's508' 'gbrg' 'grbg' 'ba81' 'rggb' 'rgb3' 'bgr3' 'jpeg' 'mjpg'(default)\n");
 		g_fprintf(fp,"mode='%s'\n",global->mode);
 		g_fprintf(fp,"# frames per sec. - hardware supported - default( %i )\n",DEFAULT_FPS);
 		g_fprintf(fp,"fps='%d/%d'\n",global->fps_num,global->fps);
@@ -72,13 +76,11 @@ writeConf(struct GLOBAL *global, char *videodevice)
 		g_fprintf(fp,"# hardware accelaration: 0 1 (default - 1)\n");
 		g_fprintf(fp,"hwaccel=%i\n",global->hwaccel);
 		g_fprintf(fp,"# video compression format: 0-MJPG 1-YUY2/UYVY 2-DIB (BMP 24) 3-MPEG1 4-FLV1 5-MPEG2 6-MS MPEG4 V3(DIV3) 7-MPEG4 (DIV5)\n");
-		g_fprintf(fp,"avi_format=%i\n",global->VidCodec);
-		g_fprintf(fp,"# video muxer: 0-avi 1-matroska\n");
-		g_fprintf(fp,"vid_mux=%i\n",global->VidFormat);
+		g_fprintf(fp,"vid_codec=%i\n",global->VidCodec);
 		g_fprintf(fp,"# avi file max size (MAX: %d bytes)\n",AVI_MAX_SIZE);
 		g_fprintf(fp,"avi_max_len=%li\n",global->AVI_MAX_LEN);
 		g_fprintf(fp,"# Auto Video naming (ex: filename-n.avi)\n");
-		g_fprintf(fp,"avi_inc=%d\n",global->vid_inc);
+		g_fprintf(fp,"vid_inc=%d\n",global->vid_inc);
 		g_fprintf(fp,"# sound 0 - disable 1 - enable\n");
 		g_fprintf(fp,"sound=%i\n",global->Sound_enable);
 		g_fprintf(fp,"# sound API: 0- Portaudio  1- Pulseaudio\n");
@@ -90,17 +92,17 @@ writeConf(struct GLOBAL *global, char *videodevice)
 		g_fprintf(fp,"# snd_numchan - sound number of channels 0- dev def 1 - mono 2 -stereo\n");
 		g_fprintf(fp,"snd_numchan=%i\n",global->Sound_NumChanInd);
 		g_fprintf(fp,"#snd_numsec - video audio blocks size in sec: 1,2,3,.. \n");
-		g_fprintf(fp,"# more seconds = more granularity, more memory allocation but less disc I/O\n");
-		g_fprintf(fp,"snd_numsec=%i\n",global->Sound_NumSec);
+		g_fprintf(fp,"# sound delay in nanosec - delays sound by the specified amount when capturing video\n");
+		g_fprintf(fp,"snd_delay=%llu\n",(unsigned long long) global->Sound_delay);
 		g_fprintf(fp,"# Sound Format (PCM=1 (0x0001) MP2=80 (0x0050)\n");
 		g_fprintf(fp,"snd_format=%i\n",global->Sound_Format);
-		g_fprintf(fp,"# Sound bit Rate used by mpeg audio default=160 Kbps\n");
-		g_fprintf(fp,"#other values: 48 56 64 80 96 112 128 160 192 224 384\n");
-		g_fprintf(fp,"snd_bitrate=%i\n",global->Sound_bitRate);
-		g_fprintf(fp,"#Pan Step in degrees, Default=2\n");
-		g_fprintf(fp,"Pan_Step=%i\n",global->PanStep);
-		g_fprintf(fp,"#Tilt Step in degrees, Default=2\n");
-		g_fprintf(fp,"Tilt_Step=%i\n",global->TiltStep);
+		//g_fprintf(fp,"# Sound bit Rate used by mpeg audio default=160 Kbps\n");
+		//g_fprintf(fp,"#other values: 48 56 64 80 96 112 128 160 192 224 384\n");
+		//g_fprintf(fp,"snd_bitrate=%i\n",global->Sound_bitRate);
+		//g_fprintf(fp,"#Pan Step in degrees, Default=2\n");
+		//g_fprintf(fp,"Pan_Step=%i\n",global->PanStep);
+		//g_fprintf(fp,"#Tilt Step in degrees, Default=2\n");
+		//g_fprintf(fp,"Tilt_Step=%i\n",global->TiltStep);
 		g_fprintf(fp,"# video filters: 0 -none 1- flip 2- upturn 4- negate 8- mono (add the ones you want)\n");
 		g_fprintf(fp,"frame_flags=%i\n",global->Frame_Flags);
 		g_fprintf(fp,"# Image capture Full Path\n");
@@ -108,17 +110,33 @@ writeConf(struct GLOBAL *global, char *videodevice)
 		g_fprintf(fp,"# Auto Image naming (filename-n.ext)\n");
 		g_fprintf(fp,"image_inc=%d\n",global->image_inc);
 		g_fprintf(fp,"# Video capture Full Path\n");
-		g_fprintf(fp,"avi_path='%s/%s'\n",global->vidFPath[1],global->vidFPath[0]);
+		g_fprintf(fp,"video_path='%s/%s'\n",global->vidFPath[1],global->vidFPath[0]);
 		g_fprintf(fp,"# control profiles Full Path\n");
 		g_fprintf(fp,"profile_path='%s/%s'\n",global->profile_FPath[1],global->profile_FPath[0]);
 		printf("write %s OK\n",global->confPath);
-		fclose(fp);
+		
+		//flush stream buffers to filesystem
+		fflush(fp);
+		
+		//close file after fsync (sync file data to disk) 
+		if (fsync(fileno(fp)) || fclose(fp))
+		{
+			perror("error writing configuration data to file");
+			ret=-1;
+		} 
+		else
+		{
+			//rename from tmp to real name
+			g_rename (tmpfile, global->confPath);
+		}
 	} 
 	else 
 	{
-		g_printerr("Could not write file %s \n Please check file permissions\n",global->confPath);
-		ret=-1;
+		g_printerr("Could not write file %s \n Please check file permissions\n",tmpfile);
+		ret=-2;
 	}
+	g_free(tmpfile);
+	
 	return ret;
 }
 
@@ -150,14 +168,14 @@ readConf(struct GLOBAL *global)
 		TRUE,                          /* scan hex */
 		FALSE,                         /* scan hex dollar */
 		TRUE,                          /* scan single quote strings */
-		TRUE,                          /* scan double quite strings */
+		TRUE,                          /* scan double quote strings */
 		TRUE,                          /* numbers to int */
 		FALSE,                         /* int to float */
 		TRUE,                          /* identifier to string */
 		TRUE,                          /* char to token */
-		FALSE,                          /* symbol to token */
+		FALSE,                         /* symbol to token */
 		FALSE,                         /* scope 0 fallback */
-		FALSE                          /* store int64 */
+		TRUE                           /* store int64 */
 	};
 
 	int fd = g_open (global->confPath, O_RDONLY, 0);
@@ -248,10 +266,14 @@ readConf(struct GLOBAL *global)
 								global->imgFormat = check_image_type(global->imgFPath[0]);
 							}
 						}
-						else if (g_strcmp0(name,"avi_path")==0) 
+						else if ((g_strcmp0(name,"video_path")==0) || (g_strcmp0(name,"avi_path")==0)) 
 						{
 							if(global->vidfile == NULL)
+							{
 								global->vidFPath=splitPath(scanner->value.v_string,global->vidFPath);
+								/*get the file type (0-avi 1-matroska)*/
+								global->VidFormat = check_video_type(global->vidFPath[0]);
+							}
 						}
 						else if (g_strcmp0(name,"profile_path")==0) 
 						{
@@ -341,20 +363,16 @@ readConf(struct GLOBAL *global)
 							if(global->flg_hwaccel < 1)
 								global->hwaccel = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"avi_format")==0) 
+						else if (g_strcmp0(name,"vid_codec")==0 || (g_strcmp0(name,"avi_format")==0)) 
 						{
 							global->VidCodec = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"vid_mux")==0) 
-						{
-							global->VidFormat = scanner->value.v_int;
-						}
 						else if (g_strcmp0(name,"avi_max_len")==0) 
 						{
 							global->AVI_MAX_LEN = (ULONG) scanner->value.v_int;
 							global->AVI_MAX_LEN = AVI_set_MAX_LEN (global->AVI_MAX_LEN);
 						}
-						else if (g_strcmp0(name,"avi_inc")==0) 
+						else if ((g_strcmp0(name,"vid_inc")==0) || (g_strcmp0(name,"avi_inc")==0)) 
 						{
 							global->vid_inc = (DWORD) scanner->value.v_int;
 							g_snprintf(global->vidinc_str,20,_("File num:%d"),global->vid_inc);
@@ -379,28 +397,28 @@ readConf(struct GLOBAL *global)
 						{
 							global->Sound_NumChanInd = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"snd_numsec")==0) 
+						else if (g_strcmp0(name,"snd_delay")==0) 
 						{
-							global->Sound_NumSec = scanner->value.v_int;
+							global->Sound_delay = scanner->value.v_int64;
 						}
 						else if (g_strcmp0(name,"snd_format")==0) 
 						{
 							global->Sound_Format = scanner->value.v_int;
 						}
-						else if (g_strcmp0(name,"snd_bitrate")==0) 
-						{
-							global->Sound_bitRate = scanner->value.v_int;
-						}
-						else if (g_strcmp0(name,"Pan_Step")==0)
-						{
-							global->PanStep = signal * scanner->value.v_int;
-							signal = 1; /*reset signal*/
-						}
-						else if (g_strcmp0(name,"Tilt_Step")==0)
-						{
-							global->TiltStep = signal * scanner->value.v_int;
-							signal = 1; /*reset signal*/
-						}
+						// else if (g_strcmp0(name,"snd_bitrate")==0) 
+						// {
+							// global->Sound_bitRate = scanner->value.v_int;
+						// }
+						//else if (g_strcmp0(name,"Pan_Step")==0)
+						//{
+						//	global->PanStep = signal * scanner->value.v_int;
+						//	signal = 1; /*reset signal*/
+						//}
+						//else if (g_strcmp0(name,"Tilt_Step")==0)
+						//{
+						//	global->TiltStep = signal * scanner->value.v_int;
+						//	signal = 1; /*reset signal*/
+						//}
 						else if (g_strcmp0(name,"frame_flags")==0) 
 						{
 							global->Frame_Flags = scanner->value.v_int;
@@ -483,9 +501,9 @@ readConf(struct GLOBAL *global)
 			g_printf("sound Device: %i\n",global->Sound_UseDev);
 			g_printf("sound samp rate: %i\n",global->Sound_SampRateInd);
 			g_printf("sound Channels: %i\n",global->Sound_NumChanInd);
-			g_printf("Sound Block Size: %i seconds\n",global->Sound_NumSec);
+			g_printf("Sound delay: %llu nanosec\n",(unsigned long long) global->Sound_delay);
 			g_printf("Sound Format: %i \n",global->Sound_Format);
-			g_printf("Sound bit Rate: %i Kbps\n",global->Sound_bitRate);
+			//g_printf("Sound bit Rate: %i Kbps\n",global->Sound_bitRate);
 			g_printf("Pan Step: %i degrees\n",global->PanStep);
 			g_printf("Tilt Step: %i degrees\n",global->TiltStep);
 			g_printf("Video Filter Flags: %i\n",global->Frame_Flags);
@@ -528,11 +546,12 @@ readOpts(int argc,char *argv[], struct GLOBAL *global)
 		{ "version", 0, 0, G_OPTION_ARG_NONE, &vers, N_("Prints version"), NULL},
 		{ "verbose", 'v', 0, G_OPTION_ARG_NONE, &global->debug, N_("Displays debug information"), NULL },
 		{ "device", 'd', 0, G_OPTION_ARG_STRING, &device, N_("Video Device to use [default: /dev/video0]"), "VIDEO_DEVICE" },
+		{ "add_ctrls", 'a', 0, G_OPTION_ARG_NONE, &global->add_ctrls, N_("Exit after adding UVC extension controls (needs root/sudo)"), NULL},
 		{ "control_only", 'o', 0, G_OPTION_ARG_NONE, &global->control_only, N_("Don't stream video (image controls only)"), NULL},
 		{ "capture_method", 'r', 0, G_OPTION_ARG_INT, &cap_meth, N_("Capture method (1-mmap (default)  2-read)"), "[1 | 2]"},
 		{ "config", 'g', 0, G_OPTION_ARG_STRING, &config, N_("Configuration file"), "FILENAME" },
 		{ "hwd_acel", 'w', 0, G_OPTION_ARG_INT, &hwaccel, N_("Hardware accelaration (enable(1) | disable(0))"), "[1 | 0]" },
-		{ "format", 'f', 0, G_OPTION_ARG_STRING, &format, N_("Pixel format(mjpg|jpeg|yuyv|yvyu|uyvy|yyuv|yu12|yv12|gbrg|grbg|bggr|rggb|bgr3|rgb3)"), "FORMAT" },
+		{ "format", 'f', 0, G_OPTION_ARG_STRING, &format, N_("Pixel format(mjpg|jpeg|yuyv|yvyu|uyvy|yyuv|yu12|yv12|nv12|nv21|nv16|nv61|y41p|grey|s501|s505|s508|gbrg|grbg|ba81|rggb|bgr3|rgb3)"), "FORMAT" },
 		{ "size", 's', 0, G_OPTION_ARG_STRING, &size, N_("Frame size, default: 640x480"), "WIDTHxHEIGHT"},
 		{ "image", 'i', 0, G_OPTION_ARG_STRING, &image, N_("Image File name"), "FILENAME"},
 		{ "cap_time", 'c', 0, G_OPTION_ARG_INT, &global->image_timer, N_("Image capture interval in seconds"), "TIME"},
@@ -540,6 +559,7 @@ readOpts(int argc,char *argv[], struct GLOBAL *global)
 		{ "video", 'n', 0, G_OPTION_ARG_STRING, &video, N_("Video File name (capture from start)"), "FILENAME"},
 		{ "vid_time", 't', 0, G_OPTION_ARG_INT, &global->Capture_time,N_("Video capture time (in seconds)"), "TIME"},
 		{ "exit_on_close", 0, 0, G_OPTION_ARG_NONE, &global->exit_on_close, N_("Exits guvcview after closing video"), NULL},
+		{ "skip", 'j', 0, G_OPTION_ARG_INT, &global->skip_n, N_("Number of inital frames to skip"), "N_FRAMES"},
 		{ "show_fps", 'p', 0, G_OPTION_ARG_INT, &FpsCount, N_("Show FPS value (enable(1) | disable (0))"), "[1 | 0]"},
 		{ "profile", 'l', 0, G_OPTION_ARG_STRING, &profile, N_("Load Profile at start"), "FILENAME"},
 		{ NULL }
@@ -671,6 +691,8 @@ readOpts(int argc,char *argv[], struct GLOBAL *global)
 		/*use fourcc but keep compatability with luvcview*/
 		if(g_strcmp0("yuv",format)==0)
 			g_snprintf(global->mode,5,"yuyv");
+		else if (g_strcmp0("bggr",format)==0) // be compatible with guvcview < 1.1.4
+			g_snprintf(global->mode,5,"ba81");
 		else
 			g_snprintf(global->mode,5,"%s",format);
 		
@@ -710,6 +732,8 @@ readOpts(int argc,char *argv[], struct GLOBAL *global)
 		global->vidfile = g_strdup(video);
 		global->vidFPath=splitPath(global->vidfile,global->vidFPath);
 		g_printf("capturing video: %s , from start",global->vidfile);
+		/*get the file type*/
+		global->VidFormat = check_video_type(global->vidFPath[0]);
 	}
 	if(profile)
 	{
