@@ -24,6 +24,10 @@
 #include <glib/gi18n.h>
 #include <glib/gprintf.h>
 #include <gtk/gtk.h>
+#include <stdio.h>
+#include <sys/types.h>
+#include <sys/statfs.h>
+
 #include "string_utils.h"
 #include "v4l2uvc.h"
 #include "globals.h"
@@ -122,3 +126,106 @@ FpsCount_callback(gpointer data)
 		return (FALSE);/*destroys the timer*/
 	}
 }
+
+/*
+ * Not a timer callback
+ * Regular function to determine if enought free space is available
+ * returns TRUE if still enough free space left on disk
+ * FALSE otherwise
+ */
+gboolean
+DiskSupervisor(gpointer data)
+{
+	struct ALL_DATA * all_data = (struct ALL_DATA *) data;
+	struct GLOBAL *global = all_data->global;
+	struct vdIn *videoIn = all_data->videoIn;
+
+	int free_tresh = 51200; //50Mb - default for compressed data
+	int percent = 0;
+	QWORD free_kbytes=0;
+	QWORD total_kbytes=0;
+	struct statfs buf;
+	
+	switch(global->VidCodec)
+	{
+		case 0: //MJPEG
+			free_tresh = 102400; // 100Mb
+			break;
+			
+		case 1: //yuyv
+			free_tresh = 358400; // 300Mb
+			break;
+			
+		case 2: //rgb
+			free_tresh = 512000; // 500Mb
+			break;
+			
+		default: //lavc
+			free_tresh = 51200; //50Mb 
+			break;
+	}
+	
+	statfs(global->vidFPath[1], &buf);
+
+	total_kbytes= buf.f_blocks * (buf.f_bsize/1024);
+	free_kbytes= buf.f_bavail * (buf.f_bsize/1024);
+
+	if(total_kbytes > 0)
+		percent = (int) ((1.0f-((float)free_kbytes/(float)total_kbytes))*100.0f);
+	else
+	{
+		g_printerr("couldn't get disk stats for %s\n", videoIn->VidFName);
+		return (TRUE); /* don't invalidate video capture*/
+	}
+	
+	if(global->debug) 
+		g_printf("(%s) %lluK bytes free on a total of %lluK (used: %d %%) treshold=%iK\n", 
+			global->vidFPath[1], (unsigned long long) free_kbytes, 
+			(unsigned long long) total_kbytes, percent, free_tresh);
+
+	
+	if(free_kbytes < free_tresh)
+	{
+		g_printerr("Not enough free disk space (%lluKb) left on disk, need > %ik \n", 
+			(unsigned long long) free_kbytes, free_tresh);
+		WARN_DIALOG(N_("Guvcview Warning:"), N_("Not enough free space left on disk"), data);
+		return(FALSE); /* not enough free space left on disk   */
+	}
+	else
+		return (TRUE); /* still have enough free space on disk */
+}
+
+/* called by video capture every 10 sec for checking disk free space*/
+gboolean 
+FreeDiskCheck_timer(gpointer data)
+{
+	struct ALL_DATA * all_data = (struct ALL_DATA *) data;
+	struct vdIn *videoIn = all_data->videoIn;
+	struct GLOBAL *global = all_data->global;
+	struct GWIDGET *gwidget = all_data->gwidget;
+	
+	g_mutex_lock(videoIn->mutex);
+		gboolean capVid = videoIn->capVid;
+	g_mutex_unlock(videoIn->mutex);
+	
+	if (capVid) 
+	{
+		if(!DiskSupervisor(data))
+		{
+			g_printerr("Stopping video Capture\n");
+			/*stop video capture*/
+			if(global->debug) g_printf("setting video toggle to FALSE\n");
+			gdk_threads_enter();
+			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(gwidget->CapVidButt), FALSE);
+			gdk_flush();
+			gdk_threads_leave();
+		}
+		else
+			return(TRUE); /*keeps the timer*/
+	}
+	
+	//global->disk_timer_id=0;
+	return (FALSE);/*destroys the timer*/
+	
+}
+
