@@ -26,13 +26,13 @@
 #include <glib/gtestutils.h>
 #include <fcntl.h>
 #include <string.h>
-#include <sys/ioctl.h>
 #include <sys/types.h>
 #include <unistd.h>
 #include <errno.h>
+#include "v4l2uvc.h"
 #include "v4l2_formats.h"
 
-#define SUP_PIX_FMT 14                //total number of software(guvcview) 
+#define SUP_PIX_FMT 23                //total number of software(guvcview) 
                                       //supported formats (list size)
 
 static SupFormats listSupFormats[SUP_PIX_FMT] = //list of software supported formats
@@ -41,11 +41,13 @@ static SupFormats listSupFormats[SUP_PIX_FMT] = //list of software supported for
 		.format   = V4L2_PIX_FMT_MJPEG,
 		.mode     = "mjpg",
 		.hardware = 0
+		//.decoder  = decode_jpeg
 	},
 	{
 		.format   = V4L2_PIX_FMT_JPEG,
 		.mode     ="jpeg",
 		.hardware = 0
+		//.decoder  = decode_jpeg
 	},
 	{
 		.format   = V4L2_PIX_FMT_YUYV,
@@ -67,6 +69,16 @@ static SupFormats listSupFormats[SUP_PIX_FMT] = //list of software supported for
 		.mode     = "yyuv",
 		.hardware = 0
 	},
+	{
+		.format   = V4L2_PIX_FMT_Y41P,
+		.mode     = "y41p",
+		.hardware = 0
+	},
+	{
+		.format   = V4L2_PIX_FMT_GREY,
+		.mode     = "grey",
+		.hardware = 0
+	},
 	{
 		.format   = V4L2_PIX_FMT_YUV420,
 		.mode     = "yu12",
@@ -77,6 +89,41 @@ static SupFormats listSupFormats[SUP_PIX_FMT] = //list of software supported for
 		.mode     = "yv12",
 		.hardware = 0
 	},
+	{
+		.format   = V4L2_PIX_FMT_NV12,
+		.mode     = "nv12",
+		.hardware = 0
+	},
+	{
+		.format   = V4L2_PIX_FMT_NV21,
+		.mode     = "nv21",
+		.hardware = 0
+	},
+	{
+		.format   = V4L2_PIX_FMT_NV16,
+		.mode     = "nv16",
+		.hardware = 0
+	},
+	{
+		.format   = V4L2_PIX_FMT_NV61,
+		.mode     = "nv61",
+		.hardware = 0
+	},
+	{
+		.format   = V4L2_PIX_FMT_SPCA501,
+		.mode     = "s501",
+		.hardware = 0
+	},
+	{
+		.format   = V4L2_PIX_FMT_SPCA505,
+		.mode     = "s505",
+		.hardware = 0
+	},
+	{
+		.format   = V4L2_PIX_FMT_SPCA508,
+		.mode     = "s508",
+		.hardware = 0
+	},
 	{
 		.format   = V4L2_PIX_FMT_SGBRG8,
 		.mode     = "gbrg",
@@ -89,7 +136,7 @@ static SupFormats listSupFormats[SUP_PIX_FMT] = //list of software supported for
 	},
 	{
 		.format   = V4L2_PIX_FMT_SBGGR8,
-		.mode     = "bggr",
+		.mode     = "ba81",
 		.hardware = 0
 	},
 	{
@@ -273,7 +320,7 @@ static int enum_frame_intervals(VidFormats *listVidFormats, __u32 pixfmt, __u32
 	listVidFormats[fmtind-1].listVidCap[fsizeind-1].framerate_denom=NULL;
 	
 	g_printf("\tTime interval between frame: ");
-	while ((ret = ioctl(fd, VIDIOC_ENUM_FRAMEINTERVALS, &fival)) == 0) 
+	while ((ret = xioctl(fd, VIDIOC_ENUM_FRAMEINTERVALS, &fival)) == 0) 
 	{
 		fival.index++;
 		if (fival.type == V4L2_FRMIVAL_TYPE_DISCRETE) 
@@ -306,8 +353,20 @@ static int enum_frame_intervals(VidFormats *listVidFormats, __u32 pixfmt, __u32
 			break;
 		}
 	}
-	
-	listVidFormats[fmtind-1].listVidCap[fsizeind-1].numb_frates = list_fps;
+
+	if (list_fps==0)
+	{
+		listVidFormats[fmtind-1].listVidCap[fsizeind-1].numb_frates = 1;
+		listVidFormats[fmtind-1].listVidCap[fsizeind-1].framerate_num = g_renew(
+				int, listVidFormats[fmtind-1].listVidCap[fsizeind-1].framerate_num, 1);
+		listVidFormats[fmtind-1].listVidCap[fsizeind-1].framerate_denom = g_renew(
+				int, listVidFormats[fmtind-1].listVidCap[fsizeind-1].framerate_denom, 1);
+			
+		listVidFormats[fmtind-1].listVidCap[fsizeind-1].framerate_num[0] = 1;
+		listVidFormats[fmtind-1].listVidCap[fsizeind-1].framerate_denom[0] = 1;
+	}
+	else
+		listVidFormats[fmtind-1].listVidCap[fsizeind-1].numb_frates = list_fps;
 	
 	g_printf("\n");
 	if (ret != 0 && errno != EINVAL) 
@@ -338,7 +397,7 @@ static int enum_frame_sizes(VidFormats *listVidFormats, __u32 pixfmt, int fmtind
 	memset(&fsize, 0, sizeof(fsize));
 	fsize.index = 0;
 	fsize.pixel_format = pixfmt;
-	while ((ret = ioctl(fd, VIDIOC_ENUM_FRAMESIZES, &fsize)) == 0) 
+	while ((ret = xioctl(fd, VIDIOC_ENUM_FRAMESIZES, &fsize)) == 0) 
 	{
 		fsize.index++;
 		if (fsize.type == V4L2_FRMSIZE_TYPE_DISCRETE) 
@@ -408,7 +467,7 @@ static int enum_frame_sizes(VidFormats *listVidFormats, __u32 pixfmt, int fmtind
 		fmt.fmt.pix.height = *height;
 		fmt.fmt.pix.pixelformat = pixfmt;
 		fmt.fmt.pix.field = V4L2_FIELD_ANY;
-		ret = ioctl(fd, VIDIOC_TRY_FMT, &fmt);
+		ret = xioctl(fd, VIDIOC_TRY_FMT, &fmt);
 		/*use the returned values*/
 		*width = fmt.fmt.pix.width;
 		*height = fmt.fmt.pix.height;
@@ -463,7 +522,7 @@ LFormats *enum_frame_formats(int *width, int *height, int fd)
 	listFormats = g_new0 ( LFormats, 1);
 	listFormats->listVidFormats = NULL;
 
-	while ((ret = ioctl(fd, VIDIOC_ENUM_FMT, &fmt)) == 0) 
+	while ((ret = xioctl(fd, VIDIOC_ENUM_FMT, &fmt)) == 0) 
 	{
 		fmt.index++;
 		g_printf("{ pixelformat = '%c%c%c%c', description = '%s' }\n",
