@@ -62,9 +62,20 @@ file_chooser (GtkButton * FileButt, struct ALL_DATA *all_data)
 		{
 			gchar *fullname = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (gwidget->FileDialog));
 			global->vidFPath=splitPath(fullname, global->vidFPath);
+			g_free(fullname);
 			gtk_entry_set_text(GTK_ENTRY(gwidget->VidFNameEntry)," ");
 			gtk_entry_set_text(GTK_ENTRY(gwidget->VidFNameEntry),global->vidFPath[0]);
-			g_free(fullname);
+			/*get the file type*/
+			global->VidFormat = check_video_type(global->vidFPath[0]);
+			/*set the file type*/
+			gtk_combo_box_set_active(GTK_COMBO_BOX(gwidget->VidFormat),global->VidFormat);
+
+			if(global->vid_inc>0)
+			{ 
+				global->vid_inc=1; /*if auto naming restart counter*/
+				g_snprintf(global->vidinc_str,24,_("File num:%d"),global->vid_inc);
+				gtk_label_set_text(GTK_LABEL(gwidget->VidIncLabel), global->vidinc_str);
+			}
 		}
 	}
 	else 
@@ -122,14 +133,29 @@ lavc_properties(GtkButton * CodecButt, struct ALL_DATA *all_data)
 		NULL);
 	
 	GtkWidget *table = gtk_table_new(1,2,FALSE);
+
+	GtkWidget *lbl_fps = gtk_label_new(_("                              encoder fps:   \n (0 - use fps combobox value)"));
+	gtk_misc_set_alignment (GTK_MISC (lbl_fps), 1, 0.5);
+	gtk_table_attach (GTK_TABLE(table), lbl_fps, 0, 1, line, line+1,
+		GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0, 0);
+	gtk_widget_show (lbl_fps);
 	
+	GtkWidget *enc_fps = gtk_spin_button_new_with_range(0,30,5);
+	gtk_editable_set_editable(GTK_EDITABLE(enc_fps),TRUE);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON(enc_fps), codec_defaults->fps);
+	
+	gtk_table_attach (GTK_TABLE(table), enc_fps, 1, 2, line, line+1,
+		GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0, 0);
+	gtk_widget_show (enc_fps);
+	line++;
+
 	GtkWidget *lbl_bit_rate = gtk_label_new(_("bit rate:   "));
 	gtk_misc_set_alignment (GTK_MISC (lbl_bit_rate), 1, 0.5);
 	gtk_table_attach (GTK_TABLE(table), lbl_bit_rate, 0, 1, line, line+1,
 		GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0, 0);
 	gtk_widget_show (lbl_bit_rate);
 	
-	GtkWidget *bit_rate = gtk_spin_button_new_with_range(160000,4000000,20000);
+	GtkWidget *bit_rate = gtk_spin_button_new_with_range(160000,4000000,10000);
 	gtk_editable_set_editable(GTK_EDITABLE(bit_rate),TRUE);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON(bit_rate), codec_defaults->bit_rate);
 	
@@ -144,7 +170,7 @@ lavc_properties(GtkButton * CodecButt, struct ALL_DATA *all_data)
 		GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0, 0);
 	gtk_widget_show (lbl_qmax);
 	
-	GtkWidget *qmax = gtk_spin_button_new_with_range(1,31,1);
+	GtkWidget *qmax = gtk_spin_button_new_with_range(1,60,1);
 	gtk_editable_set_editable(GTK_EDITABLE(qmax),TRUE);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON(qmax), codec_defaults->qmax);
 	
@@ -333,6 +359,36 @@ lavc_properties(GtkButton * CodecButt, struct ALL_DATA *all_data)
 	gtk_widget_show (qblur);
 	line++;
 
+	GtkWidget *lbl_subq = gtk_label_new(_("subq:   "));
+	gtk_misc_set_alignment (GTK_MISC (lbl_subq), 1, 0.5);
+	gtk_table_attach (GTK_TABLE(table), lbl_subq, 0, 1, line, line+1,
+		GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0, 0);
+	gtk_widget_show (lbl_subq);
+	
+	GtkWidget *subq = gtk_spin_button_new_with_range(0,8,1);
+	gtk_editable_set_editable(GTK_EDITABLE(subq),TRUE);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON(subq), codec_defaults->subq);
+	
+	gtk_table_attach (GTK_TABLE(table), subq, 1, 2, line, line+1,
+		GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0, 0);
+	gtk_widget_show (subq);
+	line++;
+
+	GtkWidget *lbl_framerefs = gtk_label_new(_("framerefs:   "));
+	gtk_misc_set_alignment (GTK_MISC (lbl_framerefs), 1, 0.5);
+	gtk_table_attach (GTK_TABLE(table), lbl_framerefs, 0, 1, line, line+1,
+		GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0, 0);
+	gtk_widget_show (lbl_framerefs);
+	
+	GtkWidget *framerefs = gtk_spin_button_new_with_range(0,12,1);
+	gtk_editable_set_editable(GTK_EDITABLE(framerefs),TRUE);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON(framerefs), codec_defaults->framerefs);
+	
+	gtk_table_attach (GTK_TABLE(table), framerefs, 1, 2, line, line+1,
+		GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0, 0);
+	gtk_widget_show (framerefs);
+	line++;
+	
 	GtkWidget *lbl_me_method = gtk_label_new(_("me method:   "));
 	gtk_misc_set_alignment (GTK_MISC (lbl_me_method), 1, 0.5);
 	gtk_table_attach (GTK_TABLE(table), lbl_me_method, 0, 1, line, line+1,
@@ -363,6 +419,21 @@ lavc_properties(GtkButton * CodecButt, struct ALL_DATA *all_data)
 	gtk_widget_show (mb_decision);
 	line++;
 	
+	GtkWidget *lbl_max_b_frames = gtk_label_new(_("max B frames:   "));
+	gtk_misc_set_alignment (GTK_MISC (lbl_max_b_frames), 1, 0.5);
+	gtk_table_attach (GTK_TABLE(table), lbl_max_b_frames, 0, 1, line, line+1,
+		GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0, 0);
+	gtk_widget_show (lbl_max_b_frames);
+	
+	GtkWidget *max_b_frames = gtk_spin_button_new_with_range(0,4,1);
+	gtk_editable_set_editable(GTK_EDITABLE(max_b_frames),TRUE);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON(max_b_frames), codec_defaults->max_b_frames);
+	
+	gtk_table_attach (GTK_TABLE(table), max_b_frames, 1, 2, line, line+1,
+		GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0, 0);
+	gtk_widget_show (max_b_frames);
+	line++;
+	
 	GtkWidget *content_area = gtk_dialog_get_content_area (GTK_DIALOG (codec_dialog));
 	gtk_container_add (GTK_CONTAINER (content_area), table);
 	gtk_widget_show (table);
@@ -371,6 +442,7 @@ lavc_properties(GtkButton * CodecButt, struct ALL_DATA *all_data)
 	switch (result)
 	{
 		case GTK_RESPONSE_ACCEPT:
+			codec_defaults->fps = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(enc_fps));
 			codec_defaults->bit_rate = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(bit_rate));
 			codec_defaults->qmax = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(qmax));
 			codec_defaults->qmin = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(qmin));
@@ -385,8 +457,11 @@ lavc_properties(GtkButton * CodecButt, struct ALL_DATA *all_data)
 			codec_defaults->gop_size = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON(gop_size));
 			codec_defaults->qcompress = gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON(qcompress));
 			codec_defaults->qblur = gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON(qblur));
+			codec_defaults->subq = gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON(subq));
+			codec_defaults->framerefs = gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON(framerefs));
 			codec_defaults->me_method = gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON(me_method));
 			codec_defaults->mb_decision = gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON(mb_decision));
+			codec_defaults->max_b_frames = gtk_spin_button_get_value_as_float (GTK_SPIN_BUTTON(max_b_frames));
 			break;
 		default:
 			// do nothing since dialog was cancelled
@@ -402,7 +477,7 @@ void video_tab(struct ALL_DATA *all_data)
 	struct GLOBAL *global = all_data->global;
 	struct vdIn *videoIn = all_data->videoIn;
 	struct GWIDGET *gwidget = all_data->gwidget;
-	struct paRecordData *pdata = all_data->pdata;
+	//struct paRecordData *pdata = all_data->pdata;
 	
 	GtkWidget *table2;
 	GtkWidget *scroll2;
@@ -410,12 +485,11 @@ void video_tab(struct ALL_DATA *all_data)
 	GtkWidget *Tab2Label;
 	GtkWidget *Tab2Icon;
 	GtkWidget *label_Device;
-	GtkWidget *FrameRate;
+	//GtkWidget *FrameRate;
 	GtkWidget *label_FPS;
 	GtkWidget *ShowFPS;
 	GtkWidget *labelResol;
-	GtkWidget *ImpType;
-	GtkWidget *label_ImpType;
+	GtkWidget *label_InpType;
 	GtkWidget *label_ImgFile;
 	GtkWidget *ImgFolder_img;
 	GtkWidget *label_ImageType;
@@ -430,6 +504,7 @@ void video_tab(struct ALL_DATA *all_data)
 	GtkWidget *FiltNegateEnable;
 	GtkWidget *FiltMonoEnable;
 	GtkWidget *FiltPiecesEnable;
+	GtkWidget *FiltParticlesEnable;
 	GtkWidget *set_jpeg_comp;
 	GtkWidget *label_jpeg_comp;
 	
@@ -513,7 +588,7 @@ void video_tab(struct ALL_DATA *all_data)
 
 	
 	if (global->debug) 
-		g_printf("resolutions of %dº format=%d \n",
+		g_printf("resolutions of format(%d) = %d \n",
 			videoIn->listFormats->current_format+1,
 			listVidFormats->numb_res);
 	for(i = 0 ; i < listVidFormats->numb_res ; i++)  
@@ -529,14 +604,11 @@ void video_tab(struct ALL_DATA *all_data)
 					defres=i;//set selected resolution index
 		}
 	}
-	
+
 	// Frame Rate
 	line++;
-	videoIn->fps_num=global->fps_num;
-	videoIn->fps=global->fps;
-	input_set_framerate (videoIn);
 					  
-	FrameRate = gtk_combo_box_new_text ();
+	gwidget->FrameRate = gtk_combo_box_new_text ();
 	int deffps=0;
 	if (global->debug) 
 		g_printf("frame rates of %dº resolution=%d \n",
@@ -544,34 +616,33 @@ void video_tab(struct ALL_DATA *all_data)
 			listVidFormats->listVidCap[defres].numb_frates);
 	for ( i = 0 ; i < listVidFormats->listVidCap[defres].numb_frates ; i++) 
 	{
-		g_snprintf(temp_str,18,"%i/%i fps", listVidFormats->listVidCap[defres].framerate_num[i],
-			listVidFormats->listVidCap[defres].framerate_denom[i]);
-		gtk_combo_box_append_text(GTK_COMBO_BOX(FrameRate),temp_str);
+		g_snprintf(temp_str,18,"%i/%i fps", listVidFormats->listVidCap[defres].framerate_denom[i],
+			listVidFormats->listVidCap[defres].framerate_num[i]);
+		gtk_combo_box_append_text(GTK_COMBO_BOX(gwidget->FrameRate),temp_str);
 		
-		if (( videoIn->fps_num == listVidFormats->listVidCap[defres].framerate_num[i]) && 
-			(videoIn->fps == listVidFormats->listVidCap[defres].framerate_denom[i]))
+		if (( global->fps_num == listVidFormats->listVidCap[defres].framerate_num[i]) && 
+			(global->fps == listVidFormats->listVidCap[defres].framerate_denom[i]))
 				deffps=i;//set selected
 	}
 	
-	gtk_table_attach(GTK_TABLE(table2), FrameRate, 1, 2, line, line+1,
+	gtk_table_attach(GTK_TABLE(table2), gwidget->FrameRate, 1, 2, line, line+1,
 		GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0, 0);
-	gtk_widget_show (FrameRate);
+	gtk_widget_show (gwidget->FrameRate);
 	
 	
-	gtk_combo_box_set_active(GTK_COMBO_BOX(FrameRate),deffps);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(gwidget->FrameRate),deffps);
 	if (deffps==0) 
 	{
-		if (listVidFormats->listVidCap[0].framerate_denom)
-			global->fps = listVidFormats->listVidCap[0].framerate_denom[0];
+		if (listVidFormats->listVidCap[defres].framerate_denom)
+			global->fps = listVidFormats->listVidCap[defres].framerate_denom[0];
 
-		if (listVidFormats->listVidCap[0].framerate_num)
-			global->fps_num = listVidFormats->listVidCap[0].framerate_num[0];
+		if (listVidFormats->listVidCap[defres].framerate_num)
+			global->fps_num = listVidFormats->listVidCap[defres].framerate_num[0];
 
-		videoIn->fps=global->fps;
-		videoIn->fps_num=global->fps_num;
+		g_printf("fps is set to %i/%i\n", global->fps_num, global->fps);
 	}
-	gtk_widget_set_sensitive (FrameRate, TRUE);
-	g_signal_connect (GTK_COMBO_BOX(FrameRate), "changed",
+	gtk_widget_set_sensitive (gwidget->FrameRate, TRUE);
+	g_signal_connect (GTK_COMBO_BOX(gwidget->FrameRate), "changed",
 		G_CALLBACK (FrameRate_changed), all_data);
 	
 	label_FPS = gtk_label_new(_("Frame Rate:"));
@@ -614,31 +685,31 @@ void video_tab(struct ALL_DATA *all_data)
 	
 	// Input Format
 	line++;
-	ImpType= gtk_combo_box_new_text ();
+	gwidget->InpType= gtk_combo_box_new_text ();
 	
 	int fmtind=0;
 	for (fmtind=0; fmtind < videoIn->listFormats->numb_formats; fmtind++)
 	{
-		gtk_combo_box_append_text(GTK_COMBO_BOX(ImpType),videoIn->listFormats->listVidFormats[fmtind].fourcc);
+		gtk_combo_box_append_text(GTK_COMBO_BOX(gwidget->InpType),videoIn->listFormats->listVidFormats[fmtind].fourcc);
 		if(global->format == videoIn->listFormats->listVidFormats[fmtind].format)
-			gtk_combo_box_set_active(GTK_COMBO_BOX(ImpType),fmtind); /*set active*/
+			gtk_combo_box_set_active(GTK_COMBO_BOX(gwidget->InpType),fmtind); /*set active*/
 	}
 	
-	gtk_table_attach(GTK_TABLE(table2), ImpType, 1, 2, line, line+1,
+	gtk_table_attach(GTK_TABLE(table2), gwidget->InpType, 1, 2, line, line+1,
 		GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0, 0);
 	
-	gtk_widget_set_sensitive (ImpType, TRUE);
-	g_signal_connect (GTK_COMBO_BOX(ImpType), "changed",
-		G_CALLBACK (ImpType_changed), all_data);
-	gtk_widget_show (ImpType);
+	gtk_widget_set_sensitive (gwidget->InpType, TRUE);
+	g_signal_connect (GTK_COMBO_BOX(gwidget->InpType), "changed",
+		G_CALLBACK (InpType_changed), all_data);
+	gtk_widget_show (gwidget->InpType);
 	
-	label_ImpType = gtk_label_new(_("Camera Output:"));
-	gtk_misc_set_alignment (GTK_MISC (label_ImpType), 1, 0.5);
+	label_InpType = gtk_label_new(_("Camera Output:"));
+	gtk_misc_set_alignment (GTK_MISC (label_InpType), 1, 0.5);
 
-	gtk_table_attach (GTK_TABLE(table2), label_ImpType, 0, 1, line, line+1,
+	gtk_table_attach (GTK_TABLE(table2), label_InpType, 0, 1, line, line+1,
 		GTK_FILL, 0, 0, 0);
 
-	gtk_widget_show (label_ImpType);
+	gtk_widget_show (label_InpType);
 
 	//jpeg compression quality for MJPEG and JPEG input formats
 	if((global->format == V4L2_PIX_FMT_MJPEG || global->format == V4L2_PIX_FMT_JPEG) && videoIn->jpgcomp.quality > 0)
@@ -778,8 +849,6 @@ void video_tab(struct ALL_DATA *all_data)
 	} 
 	else 
 	{
-		videoIn->capVid = FALSE;
-		pdata->capVid = videoIn->capVid;
 		gtk_entry_set_text(GTK_ENTRY(gwidget->VidFNameEntry),global->vidFPath[0]);
 	}
 	
@@ -934,6 +1003,21 @@ void video_tab(struct ALL_DATA *all_data)
 	g_signal_connect (GTK_CHECK_BUTTON(FiltPiecesEnable), "toggled",
 		G_CALLBACK (FiltEnable_changed), all_data);
 	
+	// Particles
+	FiltParticlesEnable=gtk_check_button_new_with_label (_(" Particles"));
+	g_object_set_data (G_OBJECT (FiltParticlesEnable), "filt_info", GINT_TO_POINTER(YUV_PARTICLES));
+	gtk_table_attach(GTK_TABLE(table_filt), FiltParticlesEnable, 1, 2, 1, 2,
+		GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0, 0);
+	
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(FiltParticlesEnable),(global->Frame_Flags & YUV_PARTICLES)>0);
+	gtk_widget_show (FiltParticlesEnable);
+	g_signal_connect (GTK_CHECK_BUTTON(FiltParticlesEnable), "toggled",
+		G_CALLBACK (FiltEnable_changed), all_data);
+	
+	
+	
+	
+	//add filters to video tab
 	gtk_table_attach (GTK_TABLE(table2), table_filt, 0, 3, line, line+1,
 		GTK_FILL, 0, 0, 0);
 	gtk_widget_show (table_filt);
